/* VbJaEngine: bitmap graphics engine for the Nintendo Virtual Boy 
 * 
 * Copyright (C) 2007 Jorge Eremiev
 * jorgech3@gmail.com
 * 
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA
 */
 
/* ---------------------------------------------------------------------------------------------------------
 * ---------------------------------------------------------------------------------------------------------
 * ---------------------------------------------------------------------------------------------------------
 * 												INCLUDES
 * ---------------------------------------------------------------------------------------------------------
 * ---------------------------------------------------------------------------------------------------------
 * ---------------------------------------------------------------------------------------------------------
 */


#include "Printing.h"
#include "FrameRate.h"
#include <libgccvb.h>


/* ---------------------------------------------------------------------------------------------------------
 * ---------------------------------------------------------------------------------------------------------
 * ---------------------------------------------------------------------------------------------------------
 * 												IMPLEMENTATIONS
 * ---------------------------------------------------------------------------------------------------------
 * ---------------------------------------------------------------------------------------------------------
 * ---------------------------------------------------------------------------------------------------------
 */

//////////////////////////////////////////////////////////////////////////////////////////////////////////////
// print hardware register's state
void printVIP_REGS(int x,int y){
	
	vbPrint(TextureManager_getFreeBgmap(TextureManager_getInstance()), x,y, "INTPND:", 0);		
	vbPrint(TextureManager_getFreeBgmap(TextureManager_getInstance()), x+8,y++, itoa((u32)VIP_REGS[INTPND],16,3), 0);
	
	vbPrint(TextureManager_getFreeBgmap(TextureManager_getInstance()), x,y, "INTENB:", 0);
	vbPrint(TextureManager_getFreeBgmap(TextureManager_getInstance()), x+8,y++, itoa((u32)VIP_REGS[INTENB],16,3), 0);
	
	vbPrint(TextureManager_getFreeBgmap(TextureManager_getInstance()), x,y, "INTCLR:", 0);
	vbPrint(TextureManager_getFreeBgmap(TextureManager_getInstance()), x+8,y++, itoa((u32)VIP_REGS[INTCLR],16,3), 0);
	
	vbPrint(TextureManager_getFreeBgmap(TextureManager_getInstance()), x,y, "DPSTTS:", 0);
	vbPrint(TextureManager_getFreeBgmap(TextureManager_getInstance()), x+8,y++, itoa((u32)VIP_REGS[DPSTTS],16,3), 0);
	
	vbPrint(TextureManager_getFreeBgmap(TextureManager_getInstance()), x,y, "DPCTRL:", 0);
	vbPrint(TextureManager_getFreeBgmap(TextureManager_getInstance()), x+8,y++, itoa((u32)VIP_REGS[DPCTRL],16,3), 0);
	
	vbPrint(TextureManager_getFreeBgmap(TextureManager_getInstance()), x,y, "FRMCYC:", 0);
	vbPrint(TextureManager_getFreeBgmap(TextureManager_getInstance()), x+8,y++, itoa((u32)VIP_REGS[FRMCYC],16,3), 0);
	
	vbPrint(TextureManager_getFreeBgmap(TextureManager_getInstance()), x,y, "CTA:", 0);
	vbPrint(TextureManager_getFreeBgmap(TextureManager_getInstance()), x+8,y++, itoa((u32)VIP_REGS[CTA],16,3), 0);
	
	vbPrint(TextureManager_getFreeBgmap(TextureManager_getInstance()), x,y, "XPSTTS:", 0);
	vbPrint(TextureManager_getFreeBgmap(TextureManager_getInstance()), x+8,y++, itoa((u32)VIP_REGS[XPSTTS],16,3), 0);
	
	vbPrint(TextureManager_getFreeBgmap(TextureManager_getInstance()), x,y, "XPCTRL:", 0);
	vbPrint(TextureManager_getFreeBgmap(TextureManager_getInstance()), x+8,y++, itoa((u32)VIP_REGS[XPCTRL],16,3), 0);
	vbPrint(TextureManager_getFreeBgmap(TextureManager_getInstance()), x,y, "GCLK:", 0);
}

//////////////////////////////////////////////////////////////////////////////////////////////////////////////
int vbjDigitCount(int value){

	int size = 0;

	do{
		value /= 10;
		size++;
		
	}while(value);

	return (size)? size: 1;
}

//////////////////////////////////////////////////////////////////////////////////////////////////////////////
void vbjPrintInt(int value,int x,int y){
		
		
	if(value < 0){
		
		value *= -1;
		
		vbPrint(TextureManager_getFreeBgmap(TextureManager_getInstance()), x++,y,"-", 0);
		
		vbPrint(TextureManager_getFreeBgmap(TextureManager_getInstance()), x,y, 
				itoa((int)(value), 10, vbjDigitCount(value)), 0);
	
	}
	else{
		vbPrint(TextureManager_getFreeBgmap(TextureManager_getInstance()), 
				//x - vbjDigitCount(value), y, itoa((int)(value), 10, vbjDigitCount(value)) + 1, 0);				
				x, y, itoa((int)(value), 10, 4), 0);
	}
}

//////////////////////////////////////////////////////////////////////////////////////////////////////////////
void vbjPrintHex(WORD value,int x,int y){
	
	if(0 && value<0){
		
		value *= -1;
		
		vbPrint(TextureManager_getFreeBgmap(TextureManager_getInstance()), x++,y,"-", 0);
		vbPrint(TextureManager_getFreeBgmap(TextureManager_getInstance()), x,y, itoa((int)(value),16,8), 0);
	
	}
	else{
		
		vbPrint(TextureManager_getFreeBgmap(TextureManager_getInstance()), x,y, itoa((int)(value),16,8), 0);
	}
}


//////////////////////////////////////////////////////////////////////////////////////////////////////////////
int vbjIntLen(int value){
	
	int length = 0;
	
	while(value > 0){
		
		value /= 10;
		
		length++;
	}
	(!length)? length++: length;
	
	return length;
}

//////////////////////////////////////////////////////////////////////////////////////////////////////////////
void vbjPrintFloat(float value,int x,int y){
	
	int sign = 1;
	
	int i = 0;
	
	int length;
	
	int size = 10;

	int decimal = (int)(((float)FIX7_9_FRAC(FTOFIX7_9(value)) / 512.f) * 100.f);
	
	if(value < 0){
		sign = -1;
		
		vbPrint(TextureManager_getFreeBgmap(TextureManager_getInstance()), x++,y,"-", 0);
	}
	
	decimal = (int)(((float)FIX7_9_FRAC(FTOFIX7_9(value * sign)) / 512.f) * 100.f);

	// print integral part
	length = vbjIntLen((int)value);
	
	vbPrint(TextureManager_getFreeBgmap(TextureManager_getInstance()), x, y, itoa(F_FLOOR(value * sign), 10, length), 0);
	
	// print the dot
	vbPrint(TextureManager_getFreeBgmap(TextureManager_getInstance()), x+length,y,".", 0);
	
	// print the decimal part
	//
	for(i = 0; size; i++){
		
		if(decimal < size){
			vbPrint(TextureManager_getFreeBgmap(TextureManager_getInstance()), x + length + 1 + i,y, itoa(0, 10, 1), 0);			
		}
		
		else{
			i++;
			break;
		}
		size /= 10;
	}
	
	
	vbPrint(TextureManager_getFreeBgmap(TextureManager_getInstance()), x + length  + i ,y, itoa(decimal, 10, 0), 0);

}

//////////////////////////////////////////////////////////////////////////////////////////////////////////////
void vbjPrintText(char *string,int x,int y){
	vbPrint(TextureManager_getFreeBgmap(TextureManager_getInstance()), x,y,string, 3);

}


//////////////////////////////////////////////////////////////////////////////////////////////////////////////
void printState(){
	

	vbjPrintText("Time:",0,0);
	FrameRate_print(FrameRate_getInstance(),0,1);
	MemoryPool_printMemUsage(MemoryPool_getInstance(),0,3);
	//GameWorld_printListsSize(_gameWorld,0,12);
	//ParamTableManager_print(_paramTableManager,0,0);
	//FrameRate_increaseCount(&gameEngine.frameRate);
	//TextureManager_print(TextureManager_getInstance() ,0, 10);
	//GameEngine_printClassesSize(&gameEngine,0,0);
	//GameWorld_printListsSize(_gameWorld,0,0);
	//Heap_printMemUsage(_heap,0,0);

}