/* VbJaEngine: bitmap graphics engine for the Nintendo Virtual Boy 
 * 
 * Copyright (C) 2007 Jorge Eremiev
 * jorgech3@gmail.com
 * 
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA
 */

#ifndef PRINTING_H_
#define	PRINTING_H_


/*---------------------------------INCLUDES--------------------------------*/


#include <libgccvb.h>
#include "MiscStructs.h"
#include "TextureManager.h"
#include "CharSetManager.h"
#include "MemoryPool.h"
#include "Globals.h"
#include <config.h>


/* ---------------------------------------------------------------------------------------------------------
 * ---------------------------------------------------------------------------------------------------------
 * ---------------------------------------------------------------------------------------------------------
 * 												PROTOTYPES
 * ---------------------------------------------------------------------------------------------------------
 * ---------------------------------------------------------------------------------------------------------
 * ---------------------------------------------------------------------------------------------------------
 */

// print VIP register's state
void printVIP_REGS(int x,int y);

// print an int
void vbjPrintInt(int value,int x,int y);

// print in hex
void vbjPrintHex(WORD value,int x,int y);

// retrieve number of digits in a number 
int vbjIntLen(int value);

// print text
void vbjPrintText(char *string,int x,int y);

// print a float
void vbjPrintFloat(float value,int x,int y);

// print engine's state
void printState();

extern Optical 		*_optical;
extern const u16 ASCII_CH[];
extern VBVec3D				*_screenPosition;

int vbjDigitCount(int value);


/* ---------------------------------------------------------------------------------------------------------
 * ---------------------------------------------------------------------------------------------------------
 * ---------------------------------------------------------------------------------------------------------
 * 											PRINTING HELPER FUNCTIONS
 * ---------------------------------------------------------------------------------------------------------
 * ---------------------------------------------------------------------------------------------------------
 * ---------------------------------------------------------------------------------------------------------
 */

//render general print output layer
inline extern  void vbjRenderOutputText(int textLayer, int textBgMap){
	
	unsigned int volatile *xpstts =	(unsigned int *)&VIP_REGS[XPSTTS];
	
	while (*xpstts & XPBSYR);
	
	//set the world's head
    WORLD_HEAD((textLayer), WRLD_ON | textBgMap);
    
    //set the world's size
    WORLD_SIZE((textLayer), 384, 224);
    
    //set the world's ...
    WORLD_GSET((textLayer), 0, __ZZERO/50, 0);
    
    //set world cuting point
    WORLD_MSET((textLayer), 0, 0, 0);

    //stop rendering at next world layer (creates a flash!)
    WORLD_HEAD((textLayer - 1), WRLD_END);
}

/* ---------------------------------------------------------------------------------------------------------*/
//setup the bgmap and char memory with printing data
inline extern  void setPrintingMemory(int bgMapSegment){
	
	//copy ascii char definition to charsegment 3
	memcpy((void*)(CharSeg3+(254*16)), (void*)_asciiChar, 258 << 4);
    //copymem((void*)(CharSeg3+(254*16)), (void*)_asciiChar, 258 << 4); //Copy chars to Char segment 3
    
    //set third char segment's mem usage
    CharSetManager_setChars(CharSetManager_getInstance(),3, 200);
    
    //clear BGMap0 but just the upper 64x28 chars    
    vbClearBGMap(bgMapSegment,1792);
}


inline extern  void  debug(u32 x){
	setPrintingMemory(TextureManager_getFreeBgmap(TextureManager_getInstance()));
	//MemoryPool_printMemUsage(MemoryPool_getInstance(),0,0);
	vbjRenderOutputText(31,TextureManager_getFreeBgmap(TextureManager_getInstance()));
	
		
	vbjPrintText("debug: ",10,10);	
	vbjPrintHex(x,17,10);
	vbjPrintInt(x,17,11);
	while(1);
}


#endif 