/* VbJaEngine: bitmap graphics engine for the Nintendo Virtual Boy 
 * 
 * Copyright (C) 2007 Jorge Eremiev
 * jorgech3@gmail.com
 * 
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA
 */

#ifndef CONSTANTS_H_
#define CONSTANTS_H_


#include "Error.h"
/* ---------------------------------------------------------------------------------------------------------
 * ---------------------------------------------------------------------------------------------------------
 * ---------------------------------------------------------------------------------------------------------
 * 							!!!!!The next values must NOT be changed!!!!!!
 * ---------------------------------------------------------------------------------------------------------
 * ---------------------------------------------------------------------------------------------------------
 * ---------------------------------------------------------------------------------------------------------
 */


//override null definition (because we don't want to include standard C libraries)
#define NULL 	(void *)0x00000000


/* ---------------------------------------------------------------------------------------------------------
 * ---------------------------------------------------------------------------------------------------------
 * ---------------------------------------------------------------------------------------------------------
 * 										MOVEMENT
 * ---------------------------------------------------------------------------------------------------------
 * ---------------------------------------------------------------------------------------------------------
 * ---------------------------------------------------------------------------------------------------------
 */

//axis definitions
#define __XYZAXIS		0x07 
#define __XYAXIS		0x03
#define __XZAXIS		0x05
#define __YZAXIS		0x06
#define __XAXIS 		0x01
#define __YAXIS 		0x02
#define __ZAXIS 		0x04


//movement type
#define	__UNIFORMMOVE	0x00
#define	__ACCELMOVEX	0x01
#define	__ACCELMOVEY	0x02
#define	__ACCELMOVEZ	0x04
#define	__RETARMOVEX	0x10
#define	__RETARMOVEY	0x20
#define	__RETARMOVEZ	0x40

//state of movement
#define __ACTIVE 		(int)0x1
#define __PASSIVE		(int)0x0

//rendering order
#define __FORWARD		0x01
#define __BACKWARD		0x02

//number of possible colliding objects
#define __OBJECTLISTTAM			32
//displacement increment to align to a colliding object
#define __COLLISIONDISP 		0.5f
#define __COLLISIONDISPY		2
#define __COLLISIONMULTIPLIER	2


/*-----------------------------posible directions--------------------------*/
#define __LEFT		 ((int)-1)
#define __RIGHT		 ((int)1)
#define __UP		 ((int)-1)
#define __DOWN		 ((int)1)
#define __NEAR		 ((int)-1)
#define __FAR		 ((int)1)

//Collision types
#define __NOCOLLISION		0x00
#define __NORMALCOLLISION	0x01
#define __INSIDEOBJECTX		0x02
#define __INSIDEOBJECTY		0x03



/* ---------------------------------------------------------------------------------------------------------
 * ---------------------------------------------------------------------------------------------------------
 * ---------------------------------------------------------------------------------------------------------
 * 										GRAPHIC MEMORY 3D
 * ---------------------------------------------------------------------------------------------------------
 * ---------------------------------------------------------------------------------------------------------
 * ---------------------------------------------------------------------------------------------------------
 */

//left buffer base addresses
#define __LEFTBUFFER1 (u32)0x00000000
#define __LEFTBUFFER2 (u32)0x00008000

//right buffer base address
#define __RIGHTBUFFER1 (u32)0x00010000
#define __RIGHTBUFFER2 (u32)0x00018000

//maximun number of solids being rendered
#define __MAXSOLIDS		16

//maximun number of vertex per polygon
#define __MAXVERTEXS	4

//maximun number of polygons per solid
#define __MAXPOLYS		16


/*-------------------------Collision logic results--------------------------*/
#define __STOPMOVEMENT		0x01
#define __NOUPDATEXPOSITION	0x02
#define __NOUPDATEYPOSITION	0x04
#define __NOUPDATEZPOSITION	0x08
#define __NOSTOPMOVEMENT	0x10
#define __NOTALIGN			0x20


/* ---------------------------------------------------------------------------------------------------------
 * ---------------------------------------------------------------------------------------------------------
 * ---------------------------------------------------------------------------------------------------------
 * 										IN GAME STATES
 * ---------------------------------------------------------------------------------------------------------
 * ---------------------------------------------------------------------------------------------------------
 * ---------------------------------------------------------------------------------------------------------
 */

enum EntityStates{
	kUnloaded = 1,
	kLoaded,
	kDead,
	kUsed,
};

/* ---------------------------------------------------------------------------------------------------------
 * ---------------------------------------------------------------------------------------------------------
 * ---------------------------------------------------------------------------------------------------------
 * 										ANIMATION TYPES
 * ---------------------------------------------------------------------------------------------------------
 * ---------------------------------------------------------------------------------------------------------
 * ---------------------------------------------------------------------------------------------------------
 */

//definition of a chargroup of an animated character or background
#define __ANIMATED			0x01
//definition of a chargroup of an unanimated character or background
#define __NO_ANIMATED		0x02
//definition of a chargroup of an animated character which it's all frames are written
//and shared
#define __ANIMATED_SHARED	0x03

// future expansion
#define __ANIMATED_SHARED_2	0x04

/* Animated character which it's chars are rewritten
 * within each frame change
 */
//#define	__WRITEANIMATED 		0x00
/* Animated character which it's param table is
 * rewriten within each frame change
 */
//#define	__WRITEANIMATEDSHARED	0x01
/* Animated character which it's param table
 * is written at one for all it's frames of animation
 */
//#define	__WRITEANIMATEDSHARED2	0x02
/* Non animated character
 */
//#define __WRITENOANIMATED		0x03

/*------------------------------ animation sizes---------------------------*/
//max leght of an animation function's name
#define __MAX_ANIMATION_FUNCTION_NAME_LENGHT	20

// max number of frames per animation function
#define __MAX_FRAMES_PER_ANIMATION_FUNCTION		16

// max number of animation functions per description
#define __MAX_ANIMATION_FUNCTIONS				32
/* ---------------------------------------------------------------------------------------------------------
 * ---------------------------------------------------------------------------------------------------------
 * ---------------------------------------------------------------------------------------------------------
 * 										IN GAME OBJECT TYPES
 * ---------------------------------------------------------------------------------------------------------
 * ---------------------------------------------------------------------------------------------------------
 * ---------------------------------------------------------------------------------------------------------
 */

//define an object type
//don't override this values with another in game type
#define __NOSOLID		1
#define __SOLID			2
#define __TRANSPARENT	3

/*--------------------------general object's types-------------------------*/
#define __NOTYPE	 	(int)0
#define __CHARACTER 	(int)1
#define __OBJCHARACTER	(int)2
#define __SCROLL 		(int)3
#define __TEXTBOX 		(int)4
#define __BACKGROUND	(int)5


/* ---------------------------------------------------------------------------------------------------------
 * ---------------------------------------------------------------------------------------------------------
 * ---------------------------------------------------------------------------------------------------------
 * 										GAME ENGINE'S STATES
 * ---------------------------------------------------------------------------------------------------------
 * ---------------------------------------------------------------------------------------------------------
 * ---------------------------------------------------------------------------------------------------------
 */

//masks to define the kind of state 
#define __MASK_PAUSE_ST		(int)0x0080
#define __MASK_RUNNING_ST	(int)0x8000

//game engine's states
#define __PRECAUTION_ST	(int)0x0000
#define __INITFOCUS_ST	(int)0x0001
#define __VBJAE_ST		(int)0x0002
#define __CREDITS_ST	(int)0x0003
#define __RESUME_ST		(int)0x0004
#define __REST_ST		(int)0x0005
#define __AUTOPAUSE_ST	(int)0x0006

/* mask every pause state with 0x0080 in order
 * to always preserve the gameplay gameworld
 */
#define __PAUSE_ST		((int)0x0000 | __MASK_PAUSE_ST)
#define __OPTIONS_ST	((int)0x0001 | __MASK_PAUSE_ST)


/* ---------------------------------------------------------------------------------------------------------
 * ---------------------------------------------------------------------------------------------------------
 * ---------------------------------------------------------------------------------------------------------
 * 										GAME ENGINE'S MESSAGES
 * ---------------------------------------------------------------------------------------------------------
 * ---------------------------------------------------------------------------------------------------------
 * ---------------------------------------------------------------------------------------------------------
 */


enum MessagesTypes{
	
	kNoCollision = 0,
	kCollisionXY,
	kCollisionX,
	kCollisionY,
	kCollisionZ,
	kFloorReached,
	kShapeBelow,
	kNoObjectBelow,
	kCollisionBelow,
	kKeyPressed,
	kEntityRemoved,
	
};




/* ---------------------------------------------------------------------------------------------------------
 * ---------------------------------------------------------------------------------------------------------
 * ---------------------------------------------------------------------------------------------------------
 * 										RENDER FLAGS
 * ---------------------------------------------------------------------------------------------------------
 * ---------------------------------------------------------------------------------------------------------
 * ---------------------------------------------------------------------------------------------------------
 */


#define __UPDATEHEAD	0x0F
#define __UPDATEG		0x01
#define __UPDATEPARAM	0x02
#define __UPDATESIZE	0x04
#define __UPDATEM		0x08

/*------------------------------ERROR-MESSAGES------------------------------*/
						   /*--------------------Max lenght-----------------*/
#define HP_MC_ERR 			"Heap: Dynamic Mem depleted (BGMap Ch)"
#define HP_OBJ_ERR 			"Heap: Dynamic Mem depleted (Object Ch)"
#define HP_SC_ERR			"Heap: Dynamic Mem depleted (Scroll)"
#define HP_BG_ERR			"Heap: Dynamic Mem depleted (Backgroud)"
#define HP_TX_ERR			"Heap: Dynamic Mem depleted (TextBox)"
#define HP_MEMDEP_ERR		"Heap: Not enought free dynamic mem"
#define HP_OBJSIZE_ERR		"Heap: object to much big"
#define HP_NOALLOC_ERR		"Heap: Deleting something not allocated"
#define HP_NOBLOCK_ERR		"Heap: Index outside memory"
				
#define BGMEM_MEM_ERR 		"BgmapMem: bgmap segments depleted"				
#define BGMEM_MC_ERR 		"BgmapMem: Mem depleted (MapCharacter)"
#define BGMEM_TX_ERR 		"BgmapMem: Mem depleted (TextBox)"
#define BGMEM_BG_ERR 		"BgmapMem: Mem depleted (Background)"
#define BGMEM_0CH_ERR		"BgmapMem: map has 0 chars"
#define BGMEM_NOALLOC_ERR	"BgmapMem: bgmap definition not found"

#define CHMEM_MEM_ERR 		"CharMem: Char Mem depleted"
#define CHMEM_0CH_ERR		"CharMem: Chargroup 0 chars"
#define OBJMEM_MEM_ERR 		"Object: Object Mem depleted"

#define INDEX_ERR 			"Index < 0"

#define VL_ENDLESS_ERR		"VirtualList: Endless list"
#define VN_EMPTY_ERR		"VirtualList: removing an empty node"

#define PT_MEM_ERR			"ParamTable: memory depleted"
#define PT_NOALLOC_ERR		"ParamTable: deallocating something null"
#define PT_OBJDEP_ERR		"ParamTable: total param objects depleted"

#define GE_NOITTXT_ERR		"GameEngine: No interactive text in gameworld"

#define GW_WLDEP_ERR		"GameWorld: world layers depleted"
#define GW_SORT_NOTYPE_ERR	"GameWorld: object has no type (sorting layers)"
#define GW_QUAD_NOOBJ_ERR	"GameWorld: object isn't in quadrant"
#define GW_NULL_CHARAC		"GameWorld: unloading a null Character"
#define GW_NULL_BACKG		"GameWorld: unloading a null Background"
#define GW_CHARAC_NOTL		"GameWorld: Character not in list"
#define GW_BACKG_NOTL		"GameWorld: Background not in list"


#define VL_ENDLESS_REM_ERR	"VirtualList: Endless list removing"
#define VL_ENDLESS_GSIZE_ERR "VirtualList: Endless list getting size"
#define VL_ENDLESS_DES_ERR	"VirtualList: Endless list destructing"
#define VL_ENDLESS_CLR_ERR	"VirtualList: Endless list clearing"
#define VL_ENDLESS_COPY_ERR	"VirtualList: Endless list copying"
#define VL_ENDLESS_PRT_ERR	"VirtualList: Endless list printing"
#define VL_NULL_DESTRUC		"VirtualList: Null destructor called"

#define CH_NULL_ACTION		"Character: invalid action called"
#define CH_NULL_COLLISION	"Character: invalid react to col called"
#define CH_NULL_NOTIFY		"Character: invalid notify called"

#define ENTITY_NULL_SPRITE	"Entity: sprite not constructed!"

#define SPRITE_NULL_TEXTURE	"Sprite: texture not constructed!"



#define NM_ASSERT( STATEMENT, ... )																\
	if(!(STATEMENT)) { 																\
		/* thrown exception */														\
		Error_triggerException(Error_getInstance(), __MAKE_STRING(__VA_ARGS__));	\
	}

#undef ASSERT

#ifndef __DEBUG
	#define ASSERT( STATEMENT, ... )

#else
	#define ASSERT( STATEMENT, ... )												\
	if(!(STATEMENT)) { 																\
		/* thrown exception */														\
		Error_triggerException(Error_getInstance(), __MAKE_STRING(__VA_ARGS__));	\
	}
#endif

#endif /*CONSTANTS_H_*/