/* VbJaEngine: bitmap graphics engine for the Nintendo Virtual Boy 
 * 
 * Copyright (C) 2007 Jorge Eremiev
 * jorgech3@gmail.com
 * 
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA
 */

#ifndef OOP_H_
#define OOP_H_

#define __MAKE_STRING(a) #a
#define __PARAMETERS(...)  __VA_ARGS__
#define __ARGUMENTS(...)  __VA_ARGS__
#define __VIRTUAL_ARGUMENTS(...)  ,__VA_ARGS__

/*--------------------------------------------------------------------------------------------------------------------------------*/
// concatenate two strigs
#define __MAKE_CONCAT(str_1,str_2) str_1 ## str_2
#define __CONCAT(str_1,str_2) __MAKE_CONCAT(str_1,str_2)



/*--------------------------------------------------------------------------------------------------------------------------------*/
// to support in run time abstract class instantiation
#define __DEFINE_CHECK_VTABLE(ClassName)								\
																		\
	/* define the checking method */									\
	void ClassName  ## _checkVTable(){									\
																		\
		/* check that each entry in the table is not NULL */			\
		int i = 0;														\
		for(; i < sizeof(ClassName ## _vTable) / sizeof(void*); i++){	\
																		\
			/* check each entry */										\
			NM_ASSERT(((void**)&ClassName ## _vTable)[i],				\
					ClassName ## : is abstract);						\
		}																\
	}

/*--------------------------------------------------------------------------------------------------------------------------------*/
#define __CHECK_VTABLE(ClassName)								\
															\
	/* setup the class's vtable on first call only */		\
	{														\
		static int __flag = false;							\
															\
		/* if not initialized */							\
		if(!__flag){										\
															\
			/* set the class vtable entries */				\
			ClassName  ## _checkVTable();					\
															\
			/* don't need to set the vtable again */		\
			__flag = true;									\
		}													\
	}


/*--------------------------------------------------------------------------------------------------------------------------------*/
#define __ALLOCATOR_DEFINITION(ClassName)							\
	ClassName ClassName ## _allocator(){							\
																	\
		/* allocate object */										\
		ClassName this = (ClassName) MemoryPool_allocate(MemoryPool_getInstance(), this, sizeof(ClassName ## _str));	\
																	\
		/* initialize the class */									\
		__SET_CLASS(ClassName);										\
																	\
		/* abstract classes can't be instantiated */				\
		__CHECK_VTABLE(ClassName);									\
																	\
		/* return constructed object's address */					\
		return this;												\
																	\
	}


/*--------------------------------------------------------------------------------------------------------------------------------*/
// define the class's allocator
#define __CLASS_NEW_DEFINITION(ClassName, ...)						\
																	\
	/* define vtable checker */										\
	__DEFINE_CHECK_VTABLE(ClassName)								\
																	\
	/* define allocator */											\
	__ALLOCATOR_DEFINITION(ClassName)								\
																	\
	/* define the method */											\
	ClassName ClassName ## _new(__VA_ARGS__){						\
																	\
		/* create the object and allocate memory for it */			\
		ClassName this = ClassName ## _allocator();			


/*--------------------------------------------------------------------------------------------------------------------------------*/
// end class's allocator
#define __CLASS_NEW_END(ClassName, ...)								\
																	\
		/* set the vtable pointer */								\
		this->vTable = &ClassName ## _vTable;						\
																	\
		/* construct the object */									\
		ClassName ## _constructor(__VA_ARGS__);						\
																	\
		/* just for safety set it again */							\
		this->vTable = &ClassName ## _vTable;						\
																	\
		/* return the created object */								\
		return this;												\
																	\
	}
	

/*--------------------------------------------------------------------------------------------------------------------------------*/
// like new in C++
#define __NEW(ClassName, ...)							\
														\
	/* call class's new implementation */				\
	ClassName ## _new(__VA_ARGS__)		
	
/* ---------------------------------------------------------------------------------------------------------*/
// like delete in C++ (calls virtual destructor)
#define __DELETE(Object)													\
																			\
	/* since the destructor is the first element in the virtual table */	\
	((void (*)(void*))((void***)Object)[0][0])(Object);		


/*--------------------------------------------------------------------------------------------------------------------------------*/
#define __DEALLOCATE(SuperClass)									\
																	\
	/* call super class's destructor */								\
	SuperClass ## _destructor((SuperClass)this);					\
																	\
	/* free the memory */											\
	MemoryPool_free(MemoryPool_getInstance(), (void*)this);			


/* ---------------------------------------------------------------------------------------------------------*/
#define __VIRTUAL_CALL_DB(ReturnType, ClassName, MethodName, Object, ...)				\
																						\
	/* call derived implementation */													\
	(((struct ClassName ## _vTable*)((*((void**)Object))))->MethodName)					\


#define __VIRTUAL_CALL(ReturnType, ClassName, MethodName, Object, ...)					\
																						\
	/* call derived implementation */													\
	((ReturnType (*)(ClassName, ...))													\
	(((struct ClassName ## _vTable*)((*((void**)Object))))->MethodName))				\
		(																				\
				Object __VA_ARGS__														\
		)		

/*--------------------------------------------------------------------------------------------------------------------------------*/
#define __VIRTUAL_METHOD(MethodName)				\
													\
	/* define a virtual method pointer */			\
	void* MethodName



/*--------------------------------------------------------------------------------------------------------------------------------*/
#define __SET_CLASS(ClassName)								\
															\
	/* setup the class's vtable on first call only */		\
	{														\
		static int __flag = false;							\
															\
		/* if not initialized */							\
		if(!__flag){										\
															\
			/* set the class vtable entries */				\
			ClassName ## _setVTable();						\
															\
			/* don't need to set the vtable again */		\
			__flag = true;									\
		}													\
	}




/*--------------------------------------------------------------------------------------------------------------------------------*/
#define __INHERITS_METHODS(SuperClass)				\
													\
	/* inherit virtual methods */					\
	__CONCAT(SuperClass,_METHODS);					\
	

#define __INHERITS_METHODS_SET(ClassName,SuperClass)			\
																\
	/*   */														\
	__CONCAT_NESTED(SuperClass,_SET_VTABLE)(ClassName);

/*--------------------------------------------------------------------------------------------------------------------------------*/
#define __CONSTRUCT_BASE(SuperClass,...)								\
																		\
	/* make sure the this pointer is initialized */						\
	ASSERT(this, SuperClass ##_constructor);							\
																		\
	/* call super constructor */										\
	SuperClass ## _constructor((SuperClass)__VA_ARGS__); 				


/*--------------------------------------------------------------------------------------------------------------------------------*/

#define __VIRTUAL_SET(TableClass, ClassName, MethodName)													\
																								\
	/* set the virtual method's address in its correspoiding vtable offset */					\
	TableClass ## _vTable.MethodName = ClassName ## _ ## MethodName

/*--------------------------------------------------------------------------------------------------------------------------------*/
#define __SET_VTABLE(ClassName, SuperClass)								\
																		\
	/* define the static method */										\
	static void ClassName ## _setVTable() 	{							\
																		\
		/* set the class's virtual methods */							\
		ClassName ## _SET_VTABLE(ClassName)								\
																		\
		/* always set the destructor at the end */						\
		__VIRTUAL_SET(ClassName, ClassName, destructor);				\
	}

/*--------------------------------------------------------------------------------------------------------------------------------*/
#define __VTABLE(ClassName)									\
															\
	/* declare the vtable struct */							\
	struct ClassName ## _vTable{							\
															\
		/* all destructors are virtual */					\
		__VIRTUAL_METHOD(destructor);						\
															\
		/* insert class's virtual methods names */			\
		ClassName ## _METHODS;								\
															\
	/* create the vtable instance */						\
	}ClassName ## _vTable;									\
															\


/*--------------------------------------------------------------------------------------------------------------------------------*/
// define a class
#define __CLASS_DEFINITION(ClassName, SuperClass)				\
																\
	typedef struct ClassName ## _str {							\
																\
		/* class attributes */									\
		ClassName ## _ATTRIBUTES								\
																\
		/* end definition */									\
	}ClassName ## _str;											\
																\
	/* define class's getSize method */							\
	int ClassName ## _getObjectSize(){							\
																\
		return sizeof(ClassName ## _str);						\
	}															\
																\
	/* now add the function which will set the vtable */		\
	__SET_VTABLE(ClassName, SuperClass)							


/*--------------------------------------------------------------------------------------------------------------------------------*/
// declare a class
#define __CLASS(ClassName)												\
																		\
	/* declare a const pointer */										\
	typedef struct ClassName ## _str* ClassName;						\
																		\
	/* declare vtable */												\
	__VTABLE(ClassName);												\
																		\
	/* declare getSize method */										\
	int ClassName ## _getObjectSize()

																		
/*--------------------------------------------------------------------------------------------------------------------------------*/
// declare an object type
#define __TYPE(ClassName)	ClassName ## _new

/*--------------------------------------------------------------------------------------------------------------------------------*/
// make a once call to this method
#define __CALL_ONCE(MethodName)						\
													\
	{												\
		/* a static flag */							\
		static int __callFlag = false;				\
													\
		/* check if not called */					\
		if(!__callFlag){							\
													\
			/* call method */						\
			MethodName();							\
		}											\
	}

/*--------------------------------------------------------------------------------------------------------------------------------*/
// defines a singleton (unique instance of a class)
#define __SINGLETON(ClassName)										\
																	\
	/* declare the static instance */								\
	static ClassName ## _str _instance ## ClassName;				\
																	\
	/* a flag to know when to allow constructs */					\
	static int _singletonConstructed = false;						\
																	\
	/* define get instance method */								\
	ClassName ClassName ## _getInstance(){							\
																	\
		/* set the vtable */										\
		__SET_CLASS(ClassName);										\
																	\
		/* first check if not constructed yet */					\
		if(!_singletonConstructed){									\
																	\
			/* call constructor */									\
			ClassName ## _constructor(&_instance ## ClassName);		\
																	\
			/* set the vtable pointer */							\
			_instance ## ClassName.vTable = &ClassName ## _vTable;	\
																	\
			/* don't allow more constructs */						\
			_singletonConstructed = true;							\
		}															\
																	\
		/* return the created singleton */							\
		return &_instance ## ClassName;								\
	}

/*--------------------------------------------------------------------------------------------------------------------------------*/
// make a once call to this method
#define __SINGLETON_DESTRUCT	_singletonConstructed = false


/*--------------------------------------------------------------------------------------------------------------------------------*/
// defines a dynamic singleton (unique instance of a class)
#define __SINGLETON_DYNAMIC(ClassName)								\
																	\
	/* declare the static pointer to instance */					\
	static ClassName _instance ## ClassName;						\
																	\
	/* define allocator */											\
	__CLASS_NEW_DEFINITION(ClassName);								\
	__CLASS_NEW_END(ClassName, this);								\
																	\
																	\
	/* a flag to know when to allow constructs */					\
	static int _singletonConstructed = false;						\
																	\
	/* define get instance method */								\
	ClassName ClassName ## _getInstance(){							\
																	\
		/* set the vtable */										\
		__SET_CLASS(ClassName);										\
																	\
		/* first check if not constructed yet */					\
		if(!_singletonConstructed){									\
																	\
			/* allocate */											\
			_instance ## ClassName = ClassName ## _new();			\
																	\
			/* don't allow more constructs */						\
			_singletonConstructed = true;							\
		}															\
																	\
		/* return the created singleton */							\
		return _instance ## ClassName;								\
	}


#endif /*OOP_H_*/
