/* VbJaEngine: bitmap graphics engine for the Nintendo Virtual Boy 
 * 
 * Copyright (C) 2007 Jorge Eremiev
 * jorgech3@gmail.com
 * 
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA
 */

/* ---------------------------------------------------------------------------------------------------------
 * ---------------------------------------------------------------------------------------------------------
 * ---------------------------------------------------------------------------------------------------------
 * 												INCLUDES
 * ---------------------------------------------------------------------------------------------------------
 * ---------------------------------------------------------------------------------------------------------
 * ---------------------------------------------------------------------------------------------------------
 */
#include "Scroll.h"
#include "Optics.h"
#include "MemoryPool.h"
#include "Globals.h"
#include "../config.h"

/* ---------------------------------------------------------------------------------------------------------
 * ---------------------------------------------------------------------------------------------------------
 * ---------------------------------------------------------------------------------------------------------
 * 											 CLASS'S MACROS
 * ---------------------------------------------------------------------------------------------------------
 * ---------------------------------------------------------------------------------------------------------
 * ---------------------------------------------------------------------------------------------------------
 */



/* ---------------------------------------------------------------------------------------------------------
 * ---------------------------------------------------------------------------------------------------------
 * ---------------------------------------------------------------------------------------------------------
 * 											CLASS'S DEFINITION
 * ---------------------------------------------------------------------------------------------------------
 * ---------------------------------------------------------------------------------------------------------
 * ---------------------------------------------------------------------------------------------------------
 */

__CLASS_DEFINITION(Scroll, Entity);

/* ---------------------------------------------------------------------------------------------------------
 * ---------------------------------------------------------------------------------------------------------
 * ---------------------------------------------------------------------------------------------------------
 * 												PROTOTYPES
 * ---------------------------------------------------------------------------------------------------------
 * ---------------------------------------------------------------------------------------------------------
 * ---------------------------------------------------------------------------------------------------------
 */

// calculate the scroll's screen position
static void Scroll_set3DPosition(Scroll this);


/* ---------------------------------------------------------------------------------------------------------
 * ---------------------------------------------------------------------------------------------------------
 * ---------------------------------------------------------------------------------------------------------
 * 												CLASS'S METHODS
 * ---------------------------------------------------------------------------------------------------------
 * ---------------------------------------------------------------------------------------------------------
 * ---------------------------------------------------------------------------------------------------------
 */

//////////////////////////////////////////////////////////////////////////////////////////////////////////////
// always call these to macros next to each other
__CLASS_NEW_DEFINITION(Scroll, __PARAMETERS(ScrollDefinition* scrollDefinition, int ID))
__CLASS_NEW_END(Scroll, __ARGUMENTS(this, scrollDefinition, ID));

//////////////////////////////////////////////////////////////////////////////////////////////////////////////
// class's constructor 
void Scroll_constructor(Scroll this, ScrollDefinition* scrollDefinition, int ID){

	__CONSTRUCT_BASE(Entity, __ARGUMENTS(this, &scrollDefinition->entityDefinition, ID));

	// create the sprites
	{
		int i = 0;
		
		for(; i < kTotalSprites; i++){
		
			this->sprites[i] = __NEW(Sprite, &scrollDefinition->spriteDefinitions[i]);
			
			// write the  to graphic memory
			__VIRTUAL_CALL(void, Sprite, write, this->sprites[i]);
		}
		
		// make sure that the Entity's sprite point to one the my sprites
		this->sprite = this->sprites[0];
	}
}


//////////////////////////////////////////////////////////////////////////////////////////////////////////////
// class's destructor
void Scroll_destructor(Scroll this){

	int i = 0;
	
	for(; i < kTotalSprites; i++){
	
		__DELETE(this->sprites[i]);
	}

	// destroy the super object
	__DEALLOCATE(Entity);
}


//////////////////////////////////////////////////////////////////////////////////////////////////////////////
// write scroll
void Scroll_write(Scroll this){
	//int i;
	//if it is the first bgmap, it has to check if there is enough memory room in the
	//bgmap segment to allocate the 3 bgmap definitions, otherwise, one of the remaining
	//bgmaps could be written in other bgmap segment and the hole thing won't work.
	//put the second map to the screen right boundary	
	//UGLY CODE!
	
	/*
	Map_write(this->map[0]);
	
	for(i = 1; i < MAPS_PER_SCROLL; i++){
		//fake char allocation
		//CharGroup_setOffset(Map_getCharGroup(&this->map[i]),0x00);
		//CharGroup_copy(Map_getCharGroup(&this->map[i]),Map_getCharGroup(&this->map[0]));
		Map_write(this->map[i]);
		//correct for ??? i don't remember (should comment on the moment of writting!!)
		//locate the x position for each map
	}
	*/
}

//////////////////////////////////////////////////////////////////////////////////////////////////////////////
// calculate the scroll's screen position
static void Scroll_set3DPosition(Scroll this){
	
	// TODO: add proper comments

	DrawSpec drawSpec0 = {
			{0, 0, 0},
			{1, 1}
	};
	
	DrawSpec drawSpec1 = drawSpec0;

	VBVec3D position3D = {_screenPosition->x, ITOFIX23_9(208), this->position.z};
	
	// get the screen's position
	VBVec2D screenPosition; 
		

	int screens = 0;

	// axis to put one map o each side of it
	int axis = 0;
	int factor = 1;
	int displacement = 0;
	
	// project position to 2D
	vbjProjectFastTo2D(&screenPosition, &position3D);
	
	// get the number of "screens" from the beginnig of the world
	// to the actual screen's position
	screens = screenPosition.x / __SCREENWIDTH;
	
	// check if the number of screens is divisible by 2
	//if(!(screens & 2)1 == 0 && screens != 0){
	displacement = screenPosition.x;
	
	if(screens){
		
		displacement -= ( screens - 1) * __SCREENWIDTH;

		if(!(screens & 1)){		
			// if so, 
			factor = 2;
		}
	}
	
	axis = __SCREENWIDTH * factor - displacement;
	
	if((unsigned)axis <= __SCREENWIDTH){
		
		drawSpec0.position.x = axis - __SCREENWIDTH - 1;
		
		drawSpec1.position.x = axis;
	}
	else{
		
		if(axis < 0){
			
			drawSpec1.position.x = axis;
			
			drawSpec0.position.x = drawSpec1.position.x + __SCREENWIDTH;
		}
		else{
			
			drawSpec0.position.x = axis - __SCREENWIDTH - 1;
			
			drawSpec1.position.x = drawSpec0.position.x - __SCREENWIDTH - 1;
		}
	}
	
	// now move the drawspec in order to render the texture in the center
	drawSpec0.position.y = drawSpec1.position.y = screenPosition.y - (Texture_getRows(Sprite_getTexture(this->sprite)) << 3);
	drawSpec0.position.parallax = drawSpec1.position.parallax = Sprite_getDrawSpec(this->sprite).position.parallax;
	
	// set map's position
	Sprite_setDrawSpec(this->sprites[kRightSprite], &drawSpec0);
	Sprite_setRenderFlag(this->sprites[kRightSprite], __UPDATEG);
	
	Sprite_setDrawSpec(this->sprites[kLeftSprite], &drawSpec1);	
	Sprite_setRenderFlag(this->sprites[kLeftSprite], __UPDATEG);
}

//////////////////////////////////////////////////////////////////////////////////////////////////////////////
// render scroll
void Scroll_render(Scroll this){

	int i = 0;
	
	if(*((int*)_screenMovementState)){
		
		// project scroll's position
		Scroll_set3DPosition(this);
	}
	
	for(i = 0; i < kTotalSprites; i++){
		
		//render map	
		__VIRTUAL_CALL(void, Sprite, render, __ARGUMENTS(this->sprites[i]));
	}
}

//////////////////////////////////////////////////////////////////////////////////////////////////////////////

//////////////////////////////////////////////////////////////////////////////////////////////////////////////
//retrieve scroll's position
/*
Position Scroll_getPosition(Scroll this){
	
	return this->position;
}
*/

//////////////////////////////////////////////////////////////////////////////////////////////////////////////
// allocate a write in graphic memory again
void Scroll_resetMemoryState(Scroll this, int worldLayer){
	
	/*
	int i = 0;
	//set map character world to zero
	for(; i < MAPS_PER_SCROLL; i++){
		
		// make sure is is allocated and written
		Map_resetMemoryState(this->map[i]);
		
		Map_setWorldLayer(this->map[i], worldLayer--);

	}
	*/
		
}

//////////////////////////////////////////////////////////////////////////////////////////////////////////////
// update sprite's world layer 
void Scroll_update(Scroll this, int worldLayer){
	
}

//////////////////////////////////////////////////////////////////////////////////////////////////////////////
// update sprite's world layer 
void Scroll_updateLayer(Scroll this, int worldLayerIncrement){
	
	int i = kTotalSprites;
	
	for(; i--;){
		
		Sprite_setWorldLayer(this->sprites[i], Sprite_getWorldLayer(this->sprites[i]) + worldLayerIncrement);
	}
	
}

//////////////////////////////////////////////////////////////////////////////////////////////////////////////
// update sprite's world layer 
void Scroll_setLayer(Scroll this, int worldLayer){

	int i = kTotalSprites;
	
	for(; i--;){
		
		Sprite_setWorldLayer(this->sprites[i], worldLayer--);
		
	}
	
	Stage_giveLayer(Game_getStage(Game_getInstance()));

}


//////////////////////////////////////////////////////////////////////////////////////////////////////////////
// set scroll's position
void Scroll_setPosition(Scroll this, VBVec3D position){
	
	// save spatial position
	this->position = position;	

	// project position
	//set scroll's position
	Scroll_set3DPosition(this);	

	// calculate sprite's parallax
	Sprite_calculateParallax(this->sprites[kLeftSprite], this->position.z);
	Sprite_calculateParallax(this->sprites[kRightSprite], this->position.z);

}

//////////////////////////////////////////////////////////////////////////////////////////////////////////////
// whether it is visible
int Scroll_isVisible(Scroll this, int pad){
	
	return true;
}

