/* VbJaEngine: bitmap graphics engine for the Nintendo Virtual Boy 
 * 
 * Copyright (C) 2007 Jorge Eremiev
 * jorgech3@gmail.com
 * 
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA
 */

#ifndef SCROLL_H_
#define SCROLL_H_


/* ---------------------------------------------------------------------------------------------------------
 * ---------------------------------------------------------------------------------------------------------
 * ---------------------------------------------------------------------------------------------------------
 * 												INCLUDES
 * ---------------------------------------------------------------------------------------------------------
 * ---------------------------------------------------------------------------------------------------------
 * ---------------------------------------------------------------------------------------------------------
 */

#include "Entity.h"


/* ---------------------------------------------------------------------------------------------------------
 * ---------------------------------------------------------------------------------------------------------
 * ---------------------------------------------------------------------------------------------------------
 * 											CLASS'S DECLARATION
 * ---------------------------------------------------------------------------------------------------------
 * ---------------------------------------------------------------------------------------------------------
 * ---------------------------------------------------------------------------------------------------------
 */

// number of map necessary to create a scroll
#define SPRITES_PER_SCROLL	2

// map indexes
enum ScrollImages{
	kRightSprite = 0,
	kLeftSprite,
	kTotalSprites,
};


#define Scroll_METHODS									\
	Entity_METHODS;
	
	

#define Scroll_SET_VTABLE(ClassName)					\
	Entity_SET_VTABLE(ClassName);						\
	__VIRTUAL_SET(ClassName, Scroll, setPosition);		\
	__VIRTUAL_SET(ClassName, Scroll, render);			\
	__VIRTUAL_SET(ClassName, Scroll, updateLayer);		\
	__VIRTUAL_SET(ClassName, Scroll, setLayer);			\
	__VIRTUAL_SET(ClassName, Scroll, isVisible);
	

// A Scroll which represent a generic object inside a Stage
__CLASS(Scroll);

#define Scroll_ATTRIBUTES							\
													\
	/* it is derivated from*/						\
	Entity_ATTRIBUTES								\
													\
	/* 2 sprites to use with this scroll */			\
	Sprite sprites[kTotalSprites];


/* ---------------------------------------------------------------------------------------------------------
 * ---------------------------------------------------------------------------------------------------------
 * ---------------------------------------------------------------------------------------------------------
 * 											CLASS'S ROM DECLARATION
 * ---------------------------------------------------------------------------------------------------------
 * ---------------------------------------------------------------------------------------------------------
 * ---------------------------------------------------------------------------------------------------------
 */

// defines a Scroll in ROM memory
typedef const struct ScrollDefinition{

	// It has an Entity at the beggining
	EntityDefinition entityDefinition;
	
	// the sprite
	SpriteDefinition spriteDefinitions[kTotalSprites];


}ScrollDefinition;
/* ---------------------------------------------------------------------------------------------------------
 * ---------------------------------------------------------------------------------------------------------
 * ---------------------------------------------------------------------------------------------------------
 * 										PUBLIC INTERFACE
 * ---------------------------------------------------------------------------------------------------------
 * ---------------------------------------------------------------------------------------------------------
 * ---------------------------------------------------------------------------------------------------------
 */

// class's allocator
Scroll Scroll_new(ScrollDefinition* scrollDefinition, int ID);

// class's constructor
void Scroll_constructor(Scroll this, ScrollDefinition* scrollDefinition, int ID);

// class's destructor
void Scroll_destructor(Scroll this);

// write scroll
void Scroll_write(Scroll this);

// render scroll
void Scroll_render(Scroll this);

// allocate a write in graphic memory again
void Scroll_resetMemoryState(Scroll this, int worldLayer);

// update sprite's world layer 
void Scroll_updateLayer(Scroll this, int worldLayerIncrement);

// update sprite's world layer 
void Scroll_setLayer(Scroll this, int worldLayer);

//set class's position
void Scroll_setPosition(Scroll this, VBVec3D position);

// whether it is visible
int Scroll_isVisible(Scroll this, int pad);


#endif /*SCROLL_H_*/
