/* VbJaEngine: bitmap graphics engine for the Nintendo Virtual Boy 
 * 
 * Copyright (C) 2007 Jorge Eremiev
 * jorgech3@gmail.com
 * 
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA
 */
 
/* ---------------------------------------------------------------------------------------------------------
 * ---------------------------------------------------------------------------------------------------------
 * ---------------------------------------------------------------------------------------------------------
 * 												INCLUDES
 * ---------------------------------------------------------------------------------------------------------
 * ---------------------------------------------------------------------------------------------------------
 * ---------------------------------------------------------------------------------------------------------
 */

#include "Shape.h"
#include "Globals.h"
#include "Optics.h"
#include "MemoryPool.h"


/* ---------------------------------------------------------------------------------------------------------
 * ---------------------------------------------------------------------------------------------------------
 * ---------------------------------------------------------------------------------------------------------
 * 											CLASS'S DEFINITION
 * ---------------------------------------------------------------------------------------------------------
 * ---------------------------------------------------------------------------------------------------------
 * ---------------------------------------------------------------------------------------------------------
 */

// define the Sprite
__CLASS_DEFINITION(Shape, Object)


/* ---------------------------------------------------------------------------------------------------------
 * ---------------------------------------------------------------------------------------------------------
 * ---------------------------------------------------------------------------------------------------------
 * 												PROTOTYPES
 * ---------------------------------------------------------------------------------------------------------
 * ---------------------------------------------------------------------------------------------------------
 * ---------------------------------------------------------------------------------------------------------
 */




/* ---------------------------------------------------------------------------------------------------------
 * ---------------------------------------------------------------------------------------------------------
 * ---------------------------------------------------------------------------------------------------------
 * 												CLASS'S METHODS
 * ---------------------------------------------------------------------------------------------------------
 * ---------------------------------------------------------------------------------------------------------
 * ---------------------------------------------------------------------------------------------------------
 */


//////////////////////////////////////////////////////////////////////////////////////////////////////////////
// class's constructor
void Shape_constructor(Shape this, InGameEntity owner, int deep){

	// this is an abstract class so must initialize the vtable here
	// since this class does not have an allocator
	__SET_CLASS(Shape);	

	// construct base object
	__CONSTRUCT_BASE(Object,__ARGUMENTS(this));

	// set the owner	
	this->owner = owner;
	
	// set the deep
	this->deep = deep;
	
	// it is active
	this->active = true;
	
	// do I move?
	this->moves = __VIRTUAL_CALL(int, InGameEntity, moves, owner);
	
	// not checked yet
	this->checked = false;
}

//////////////////////////////////////////////////////////////////////////////////////////////////////////////
// class's destructor
void Shape_destructor(Shape this){

	// destroy the super object
	__DEALLOCATE(Object);

}

//////////////////////////////////////////////////////////////////////////////////////////////////////////////
// retrieve owner
InGameEntity Shape_getOwner(Shape this){
	
	return this->owner;
}

//////////////////////////////////////////////////////////////////////////////////////////////////////////////
// set active
void Shape_setActive(Shape this, int active){
	
	// it is active
	this->active = active;

}

//////////////////////////////////////////////////////////////////////////////////////////////////////////////
// is active?
int Shape_isActive(Shape this){
	
	return this->active;
}

//////////////////////////////////////////////////////////////////////////////////////////////////////////////
// do I move?
int Shape_moves(Shape this){
	
	return this->moves;
}

//////////////////////////////////////////////////////////////////////////////////////////////////////////////
// has been checked
int Shape_isChecked(Shape this){
	
	return this->checked;
}

//////////////////////////////////////////////////////////////////////////////////////////////////////////////
// set check status
void Shape_checked(Shape this, int checked){
	
	this->checked = checked;
}
