/* VbJaEngine: bitmap graphics engine for the Nintendo Virtual Boy 
 * 
 * Copyright (C) 2007 Jorge Eremiev
 * jorgech3@gmail.com
 * 
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA
 */



/*---------------------------------INCLUDES--------------------------------*/
#include "Bridge.h"

#include "textures.h"


/* ---------------------------------------------------------------------------------------------------------
 * ---------------------------------------------------------------------------------------------------------
 * ---------------------------------------------------------------------------------------------------------
 * 											 CLASS'S DEFINITION
 * ---------------------------------------------------------------------------------------------------------
 * ---------------------------------------------------------------------------------------------------------
 * ---------------------------------------------------------------------------------------------------------
 */


// Bridge.c

// A Bridge!
__CLASS_DEFINITION(Bridge, Background); 

/* ---------------------------------------------------------------------------------------------------------
 * ---------------------------------------------------------------------------------------------------------
 * ---------------------------------------------------------------------------------------------------------
 * 												PROTOTYPES
 * ---------------------------------------------------------------------------------------------------------
 * ---------------------------------------------------------------------------------------------------------
 * ---------------------------------------------------------------------------------------------------------
 */


/* ---------------------------------------------------------------------------------------------------------
 * ---------------------------------------------------------------------------------------------------------
 * ---------------------------------------------------------------------------------------------------------
 * 												CLASS'S METHODS
 * ---------------------------------------------------------------------------------------------------------
 * ---------------------------------------------------------------------------------------------------------
 * ---------------------------------------------------------------------------------------------------------
 */


//////////////////////////////////////////////////////////////////////////////////////////////////////////////
// always call these to macros next to each other
__CLASS_NEW_DEFINITION(Bridge, __PARAMETERS(BridgeDefinition* bridgeDefinition, u8 ID))
__CLASS_NEW_END(Bridge, __ARGUMENTS(this, bridgeDefinition, ID));

//////////////////////////////////////////////////////////////////////////////////////////////////////////////
// class's conctructor
void Bridge_constructor(Bridge this, BridgeDefinition* bridgeDefinition, int inGameIndex){
	
	__CONSTRUCT_BASE(Background, __ARGUMENTS(this, &bridgeDefinition->backgroundDefinition, inGameIndex));

	this->zDirection = bridgeDefinition->zDirection;
}


//////////////////////////////////////////////////////////////////////////////////////////////////////////////
// class's conctructor
void Bridge_destructor(Bridge this){

	// delete the super object
	__DEALLOCATE(Background);
}


//////////////////////////////////////////////////////////////////////////////////////////////////////////////
// retrieve the direction
s8 Bridge_getDirection(Bridge this){
	
	return this->zDirection;
}

//////////////////////////////////////////////////////////////////////////////////////////////////////////////
// set the direction
void Bridge_setDirection(Bridge this, s8 zDirection){
	
	this->zDirection = zDirection;
}





BridgeDefinition BRIDGE_FAR_BG = {
		{
			{
				{
					// object's class			   
					__TYPE(Bridge),  
			
					// Sprite
					{
						// the texture
						&BRIDGE0_TX,
						
						// bgmap mode ( BGMAP, AFFINE, H-BIAS)
						WRLD_AFFINE,
						
						// display mode
						WRLD_ON,
					},
				},				
				// deep
				5,
				
				// friction factor
				5.5,
				
				//collision detection gap			
				//up,	down,	left,	right,
				{0,		0,		16,		16,},
				
				// in game type
				kBridge,
			},
	
			// register shape
			true,
		},
		
		// the direction over which mario can walk on it
		__FAR,
};

BridgeDefinition BRIDGE_NEAR_BG = {
		
		{
			{
				{
					// object's class			   
					__TYPE(Bridge),  
			
					// Sprite
					{
						// the texture
						&BRIDGE0_TX,
						
						// bgmap mode ( BGMAP, AFFINE, H-BIAS)
						WRLD_AFFINE,
						
						// display mode
						WRLD_ON,
					},
				},				
				// deep
				5,
				
				// friction factor
				5.5,
				
				//collision detection gap			
				//up,	down,	left,	right,
				{0,		0,		14,		14,},
				
				// in game type
				kBridge,
			},
	
			// register shape
			true,
		},

		
		// the direction over which mario can walk on it
		__NEAR,
};

