/* VbJaEngine: bitmap graphics engine for the Nintendo Virtual Boy 
 * 
 * Copyright (C) 2007 Jorge Eremiev
 * jorgech3@gmail.com
 * 
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA
 */

#include <libgccvb2/libgccvb.h>
#include "textures.h"
#include "../../macros.h"

/* ---------------------------------------------------------------------------------------------------------*/
/* ---------------------------------------------------------------------------------------------------------*/
ImageDefinition PVBCC_LEFT_BG = {
		{
			// object's class			   
			__TYPE(Image),  
	
			// Sprite
			{
				// the texture
				&PVBCC_LEFT_TX,
				
				// bgmap mode ( BGMAP, AFFINE, H-BIAS)
				WRLD_BGMAP,
				
				// display mode
				WRLD_LON,
			},
		},
};


//////////////////////////////////////////////////////////////////////////////////////////////////////////////
ImageDefinition PVBCC_RIGHT_BG = {
		{
			// object's class			   
			__TYPE(Image),  
	
			// Sprite
			{
				// the texture
				&PVBCC_RIGHT_TX,
				
				// bgmap mode ( BGMAP, AFFINE, H-BIAS)
				WRLD_BGMAP,
				
				// display mode
				WRLD_RON,
			},
		},
};

//////////////////////////////////////////////////////////////////////////////////////////////////////////////
ImageDefinition PRECAUTION_BG = {
		{
			// object's class			   
			__TYPE(Image),  
	
			// Sprite
			{
				// the texture
				&PRECAUTION_TX,
				
				// bgmap mode ( BGMAP, AFFINE, H-BIAS)
				WRLD_BGMAP,
				
				// display mode
				WRLD_ON,
			},
		},
};

//////////////////////////////////////////////////////////////////////////////////////////////////////////////
ImageDefinition PVB_LEFT_BG = {
		{
			// object's class			   
			__TYPE(Image),  
	
			// Sprite
			{
				// the texture
				&PVB_LEFT_TX,
				
				// bgmap mode ( BGMAP, AFFINE, H-BIAS)
				WRLD_BGMAP,
				
				// display mode
				WRLD_LON,
			},
		},
};

//////////////////////////////////////////////////////////////////////////////////////////////////////////////
ImageDefinition PVB_RIGHT_BG = {
		{
			// object's class			   
			__TYPE(Image),  
	
			// Sprite
			{
				// the texture
				&PVB_RIGHT_TX,
				
				// bgmap mode ( BGMAP, AFFINE, H-BIAS)
				WRLD_BGMAP,
				
				// display mode
				WRLD_RON,
			},
		},
};
//////////////////////////////////////////////////////////////////////////////////////////////////////////////
ImageDefinition VBJAE_LEFT_BG = {
		{
			// object's class			   
			__TYPE(Image),  
	
			// Sprite
			{
				// the texture
				&VBJAE_LEFT_TX,
				
				// bgmap mode ( BGMAP, AFFINE, H-BIAS)
				WRLD_BGMAP,
				
				// display mode
				WRLD_LON,
			},
		},
};

//////////////////////////////////////////////////////////////////////////////////////////////////////////////
ImageDefinition VBJAE_RIGHT_BG = {
		{
			// object's class			   
			__TYPE(Image),  
	
			// Sprite
			{
				// the texture
				&VBJAE_RIGHT_TX,
				
				// bgmap mode ( BGMAP, AFFINE, H-BIAS)
				WRLD_BGMAP,
				
				// display mode
				WRLD_RON,
			},
		},
};


//////////////////////////////////////////////////////////////////////////////////////////////////////////////
BackgroundDefinition FLOOR0_BG = {
		{
			{
				// object's class			   
				__TYPE(Background),  
		
				// Sprite
				{
					// the texture
					&FLOOR0_TX,
					
					// bgmap mode ( BGMAP, AFFINE, H-BIAS)
					WRLD_BGMAP,
					
					// display mode
					WRLD_ON,
				},
			},
			
			// deep
			15,
			
			// friction factor
			FTOFIX7_9(10.0),
			
			//collision detection gap			
			//up,	down,	left,	right,
			{1,		0,		0,		0,},
			
			// in game type
			kSolid,
		},

		// register shape
		true,
		
};

//////////////////////////////////////////////////////////////////////////////////////////////////////////////
BackgroundDefinition FLOOR1_BG = {
		{
			{
				// object's class			   
				__TYPE(Background),  
		
				// Sprite
				{
					// the texture
					&FLOOR1_TX,
					
					// bgmap mode ( BGMAP, AFFINE, H-BIAS)
					WRLD_BGMAP,
					
					// display mode
					WRLD_ON,
				},
			},
			
			// deep
			15,
			
			// friction factor
			FTOFIX7_9(10.0),
			
			//collision detection gap			
			//up,	down,	left,	right,
			{1,		0,		0,		0},
			
			// in game type
			kSolid,
		},

		// register shape
		true,
		
};

//////////////////////////////////////////////////////////////////////////////////////////////////////////////
BackgroundDefinition FLOOR2_BG = {
		{
			{
				// object's class			   
				__TYPE(Background),  
		
				// Sprite
				{
					// the texture
					&FLOOR2_TX,
					
					// bgmap mode ( BGMAP, AFFINE, H-BIAS)
					WRLD_BGMAP,
					
					// display mode
					WRLD_ON,
				},
			},
			
			// deep
			15,
			
			// friction factor
			FTOFIX7_9(10.0),
			
			//collision detection gap			
			//up,	down,	left,	right,
			{1,		0,		0,		0},
			
			// in game type
			kSolid,
		},

		// register shape
		true,
		
};

//////////////////////////////////////////////////////////////////////////////////////////////////////////////
ImageDefinition MOUNT0_BG = {
		{
			// object's class			   
			__TYPE(Image),  
	
			// Sprite
			{
				// the texture
				&MOUNT0_TX,
				
				// bgmap mode ( BGMAP, AFFINE, H-BIAS)
				WRLD_BGMAP,
				
				// display mode
				WRLD_ON,
			},
		},

};


//////////////////////////////////////////////////////////////////////////////////////////////////////////////
ImageDefinition TRANSPARENTBRIDGE_BG = {
		{
			// object's class			   
			__TYPE(Image),  
	
			// Sprite
			{
				// the texture
				&BRIDGE0_TX,
				
				// bgmap mode ( BGMAP, AFFINE, H-BIAS)
				WRLD_AFFINE,
				
				// display mode
				WRLD_ON,
			},
		},
};

//////////////////////////////////////////////////////////////////////////////////////////////////////////////
ImageDefinition MOUNTS0_BG = {
		
		{
			// object's class			   
			__TYPE(Image),  
	
			// Sprite
			{
				// the texture
				&MOUNTS0_TX,
				
				// bgmap mode ( BGMAP, AFFINE, H-BIAS)
				WRLD_BGMAP,
				
				// display mode
				WRLD_ON,
			},
		},		
};


//////////////////////////////////////////////////////////////////////////////////////////////////////////////
ImageDefinition JAPANARC_BG = {
		{
			// object's class			   
			__TYPE(Image),  
	
			// Sprite
			{
				// the texture
				&JAPANARC_TX,
				
				// bgmap mode ( BGMAP, AFFINE, H-BIAS)
				WRLD_BGMAP,
				
				// display mode
				WRLD_ON,
			},
		},
};

//////////////////////////////////////////////////////////////////////////////////////////////////////////////
BackgroundDefinition PIPE0_BG = {
		{
			{
				// object's class			   
				__TYPE(Background),  
		
				// Sprite
				{
					// the texture
					&PIPE0_TX,
					
					// bgmap mode ( BGMAP, AFFINE, H-BIAS)
					WRLD_AFFINE,
					
					// display mode
					WRLD_ON,
				},
			},
			
			// deep
			20,
			
			// friction factor
			FTOFIX7_9(12.0),
			
			//collision detection gap			
			//up,	down,	left,	right,
			{0,		0,		0,		4,},
			
			// in game type
			kSolid,
		},

		// register shape
		true,
			
};

//////////////////////////////////////////////////////////////////////////////////////////////////////////////
BackgroundDefinition PIPE1_BG = {
		{
			{
				// object's class			   
				__TYPE(Background),  
		
				// Sprite
				{
					// the texture
					&PIPE1_TX,
					
					// bgmap mode ( BGMAP, AFFINE, H-BIAS)
					WRLD_AFFINE,
					
					// display mode
					WRLD_ON,
				},
			},
			
			// deep
			20,
			
			// friction factor
			FTOFIX7_9(12.0),
			
			//collision detection gap			
			//up,	down,	left,	right,
			{0,		0,		0,		0,},
			
			// in game type
			kSolid,
		},

		// register shape
		true,
			
};
//////////////////////////////////////////////////////////////////////////////////////////////////////////////
BackgroundDefinition WALL0_BG = {
		{
			{
				// object's class			   
				__TYPE(Background),  
		
				// Sprite
				{
					// the texture
					&WALL0_TX,
					
					// bgmap mode ( BGMAP, AFFINE, H-BIAS)
					WRLD_AFFINE,
					
					// display mode
					WRLD_ON,
				},
			},
			
			// deep
			15,
			
			// friction factor
			FTOFIX7_9(2.5),
			
			//collision detection gap			
			//up,	down,	left,	right,
			{1,		1,		0,		0,},
			
			// in game type
			kSolid,
		},

		// register shape
		true,
			
};

//////////////////////////////////////////////////////////////////////////////////////////////////////////////
BackgroundDefinition NORMALBLOCK_BG = {
		{
			{
				// object's class			   
				__TYPE(Background),  
		
				// Sprite
				{
					// the texture
					&NORMALBLOCK_TX,
					
					// bgmap mode ( BGMAP, AFFINE, H-BIAS)
					WRLD_AFFINE,
					
					// display mode
					WRLD_ON,
				},
			},
			
			// deep
			5,
			
			// friction factor
			FTOFIX7_9(12.5),
			
			//collision detection gap			
			//up,	down,	left,	right,
			{0,		4,		0,		0,},
			
			// in game type
			kSolid,
		},

		// register shape
		true,
};

//////////////////////////////////////////////////////////////////////////////////////////////////////////////
ImageDefinition GOAL0_BG = {
		{
			// object's class			   
			__TYPE(Image),  
	
			// Sprite
			{
				// the texture
				&GOAL0_TX,
				
				// bgmap mode ( BGMAP, AFFINE, H-BIAS)
				WRLD_BGMAP,
				
				// display mode
				WRLD_ON,
			},
		},
};

BackgroundDefinition POLE0_BG = {
		{
			{
				// object's class			   
				__TYPE(Background),  
		
				// Sprite
				{
					// the texture
					&POLE0_TX,
					
					// bgmap mode ( BGMAP, AFFINE, H-BIAS)
					WRLD_BGMAP,
					
					// display mode
					WRLD_ON,
				},
			},
			
			// deep
			10,
			
			// friction factor
			12.5,
			
			//collision detection gap			
			//up,	down,	left,	right,
			{0,		0,		0,		0,},
			
			// in game type
			kGoal,
		},

		// register shape
		true,
};


BackgroundDefinition GATE_BG = {
		{
			{
				// object's class			   
				__TYPE(Background),  
		
				// Sprite
				{
					// the texture
					&GATE_TX,
					
					// bgmap mode ( BGMAP, AFFINE, H-BIAS)
					WRLD_BGMAP,
					
					// display mode
					WRLD_ON,
				},
			},
			
			// deep
			10,
			
			// friction factor
			12.5,
			
			//collision detection gap			
			//up,	down,	left,	right,
			{0,		0,		0,		0,},
			
			// in game type
			kGate,
		},

		// register shape
		true,
};


//////////////////////////////////////////////////////////////////////////////////////////////////////////////
ImageDefinition MARIO_TITLE_LEFT_BG = {
		{
			// object's class			   
			__TYPE(Image),  
	
			// Sprite
			{
				// the texture
				&MARIO_TITLE_LEFT_TX,
				
				// bgmap mode ( BGMAP, AFFINE, H-BIAS)
				WRLD_BGMAP,
				
				// display mode
				WRLD_LON,
			},
		},
};



//////////////////////////////////////////////////////////////////////////////////////////////////////////////
ImageDefinition MARIO_TITLE_RIGHT_BG = {
		{
			// object's class			   
			__TYPE(Image),  
	
			// Sprite
			{
				// the texture
				&MARIO_TITLE_RIGHT_TX,
				
				// bgmap mode ( BGMAP, AFFINE, H-BIAS)
				WRLD_BGMAP,
				
				// display mode
				WRLD_RON,
			},
		},
};

//////////////////////////////////////////////////////////////////////////////////////////////////////////////
ImageDefinition TITLE_BG = {
		{
			// object's class			   
			__TYPE(Image),  
	
			// Sprite
			{
				// the texture
				&TITLE_TX,
				
				// bgmap mode ( BGMAP, AFFINE, H-BIAS)
				WRLD_BGMAP,
				
				// display mode
				WRLD_ON,
			},
		},
};

/*

EntityDefinition BIGPOW_BG = {
// 		inGameType			   
		__SOLID,  	
// 		deep
		10,		
//		friction factor
		1,
//RENDER			   
//		chDefinition,				 numChars,
		BIGPOW_CH,					  	8,
//		bgmapDef,					 cols,			  rows,			  	  mapMode,
		BIGPOW_MP,					4,				 4,		  	   WRLD_AFFINE,
//		display,			pallet number,
		WRLD_ON,						1,
//		fx,						animation
  		Map_scaleAFX,		
 		 //collision detection gap			
 		 //		up,		down,	left,	right,
 		{0,		0,		0,		0,}
};





EntityDefinition MOUNT2_BG = {
// 		inGameType			   
		__SOLID,  	
// 		deep
		10,		
//		friction factor
		1,

//RENDER			   
//		chDefinition,				 numChars,
		MOUNT2_CH,					  18,
//		bgmapDef,					 cols,			  rows,			  	  mapMode,
		MOUNT2_MP,					4,				 5,		  	   WRLD_BGMAP,
//		display,			pallet number,
		WRLD_ON,						1,
//		fx,						animation
  		NULL,		
 		 //collision detection gap			
 		 //		up,		down,	left,	right,
 		{0,		0,		0,		0,}
};








EntityDefinition POW0_BG = {
// 		inGameType			   
		__SOLID,  	
// 		deep
		10,		
//		friction factor
		1,

//RENDER			   
//		chDefinition,				 numChars,
		POW_CH,					    3,
//		bgmapDef,					 cols,			  rows,			  	  mapMode,
		POW_MP,					2,				 2,		  	   WRLD_BGMAP,
//		display,			pallet number,
		WRLD_ON,						1,
//		fx,						animation
  		Map_scaleAFX,		
 		 //collision detection gap			
 		 //		up,		down,	left,	right,
 		{0,		0,		0,		0,}
};







EntityDefinition FLOOR1_BG = {
// 		inGameType			   
		__SOLID,  	
// 		deep
		10,		
//		friction factor
		5.5,
//RENDER			   
//		chDefinition,				 numChars,
		FLOOR1_CH,					   18,
//		bgmapDef,					 cols,			  rows,			  	  mapMode,
		FLOOR1_MP,					   60,				 3,		  	   WRLD_BGMAP,
//		display,			pallet number,
		WRLD_ON,						1,
//		fx,						animation
  		NULL,		
 		 //collision detection gap			
 		 //		up,		down,	left,	right,
 		{0,		0,		0,		0,}
};


EntityDefinition FLOOR2_BG = {
// 		inGameType			   
		__SOLID,  	
// 		deep
		10,		
//		friction factor
		5.5,

//RENDER			   
//		chDefinition,				 numChars,
		FLOOR2_CH,					   11,
//		bgmapDef,					 cols,			  rows,			  	  mapMode,
		FLOOR2_MP,				   14,				 12,		  	   WRLD_BGMAP,
//		display,			pallet number,
		WRLD_ON,						1,
//		fx,						animation
  		NULL,		
 		 //collision detection gap			
 		 //		up,		down,	left,	right,
 		{0,		0,		0,		0,}
};

EntityDefinition MOUNTS0_BG = {
// 		inGameType			   
		__NOSOLID,  	
// 		deep
		10,		
//		friction factor
		1,

//RENDER			   
//		chDefinition,				 numChars,
		MOUNTS0_CH,				   18,
//		bgmapDef,					 cols,			  rows,			  	  mapMode,
		MOUNTS0_MP,				12,				 5,		  	   WRLD_BGMAP,
//		display,			pallet number,
		WRLD_ON,						1,
//		fx,						animation
  		NULL,		
 		 //collision detection gap			
 		 //		up,		down,	left,	right,
 		{0,		0,		0,		0,}
};

EntityDefinition MOUNTS1_BG = {
// 		inGameType			   
		__NOSOLID,  	
// 		deep
		10,		
//		friction factor
		1,

//RENDER			   
//		chDefinition,				 numChars,
		MOUNTS1_CH,				   83,
//		bgmapDef,					 cols,			  rows,			  	  mapMode,
		MOUNTS1_MP,			   27,				 6,		  	   WRLD_BGMAP,
//		display,			pallet number,
		WRLD_ON,						1,
//		fx,						animation
  		NULL,		
 		 //collision detection gap			
 		 //		up,		down,	left,	right,
 		{0,		0,		0,		0,}
};



};
*/