/* VbJaEngine: bitmap graphics engine for the Nintendo Virtual Boy 
 * 
 * Copyright (C) 2007 Jorge Eremiev
 * jorgech3@gmail.com
 * 
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA
 */
#ifndef KOOPA_H_
#define KOOPA_H_

/* ---------------------------------------------------------------------------------------------------------
 * ---------------------------------------------------------------------------------------------------------
 * ---------------------------------------------------------------------------------------------------------
 * 												INCLUDES
 * ---------------------------------------------------------------------------------------------------------
 * ---------------------------------------------------------------------------------------------------------
 * ---------------------------------------------------------------------------------------------------------
 */


#include "../../../macros.h"
#include "../enemy/Enemy.h"

/* ---------------------------------------------------------------------------------------------------------
 * ---------------------------------------------------------------------------------------------------------
 * ---------------------------------------------------------------------------------------------------------
 * 											CLASS'S DECLARATION
 * ---------------------------------------------------------------------------------------------------------
 * ---------------------------------------------------------------------------------------------------------
 * ---------------------------------------------------------------------------------------------------------
 */

#define KOOPA_RECOVER_TIME			10000 // milisecons
#define KOOPA_VELOCITY_X 			ITOFIX23_9(30)
#define KOOPA_VELOCITY_Y			ITOFIX23_9(100)
#define KOOPA_VELOCITY_Z			ITOFIX23_9(10)
#define KOOPA_ACCELERATION_X		ITOFIX23_9(100)
#define KOOPA_ACCELERATION_Y		ITOFIX23_9(100)
#define KOOPA_ACCELERATION_Z		ITOFIX23_9(0)
#define KOOPA_BOUNCE_VELOCITY_X		ITOFIX23_9(180)
#define KOOPA_BOUNCE_VELOCITY_Z		ITOFIX23_9(195)

#define Koopa_METHODS							\
	Enemy_METHODS;
	
	

#define Koopa_SET_VTABLE(ClassName)						\
	Enemy_SET_VTABLE(ClassName);						\
	__VIRTUAL_SET(ClassName, Koopa, die);				\
	__VIRTUAL_SET(ClassName, Koopa, beThrown);			\
	__VIRTUAL_SET(ClassName, Koopa, canAttack);			\
	__VIRTUAL_SET(ClassName, Koopa, takeHit);
	
	

// A Decoration which represents a generic object inside a Stage
__CLASS(Koopa);

#define Koopa_ATTRIBUTES							\
													\
	/* it is derivated from */						\
	Enemy_ATTRIBUTES



/* ---------------------------------------------------------------------------------------------------------
 * ---------------------------------------------------------------------------------------------------------
 * ---------------------------------------------------------------------------------------------------------
 * 										PUBLIC INTERFACE
 * ---------------------------------------------------------------------------------------------------------
 * ---------------------------------------------------------------------------------------------------------
 * ---------------------------------------------------------------------------------------------------------
 */

Koopa Koopa_new(ActorDefinition* animatedEntityDefinition, int ID);

//class's conctructor
void Koopa_constructor(Koopa this, ActorDefinition* definition, int ID);

// class's conctructor
void Koopa_destructor(Koopa this);

// register a shape with the collision detection system
void Koopa_registerShape(Koopa this);

// update movement
void Koopa_walk(Koopa this);

// update movement
void Koopa_slide(Koopa this);

// process a collision
u8 Koopa_processCollision(Koopa this, Telegram telegram);

// tell me I've been hit
void Koopa_takeHit(Koopa this, int axis, s8 direction);

// go back to normal
void Koopa_recover(Koopa this);

// begin bouncing in a given direction
void Koopa_bounce(Koopa this, s8 direction);

// retrieve graphical gap
void Koopa_setGap(Koopa this);

// call when turn animation is done
void Koopa_turnComplete(Koopa this);

// attack logic
void Koopa_attack(Koopa this);

// check if turn around
int Koopa_checkIfTurn(Koopa this, InGameEntity inGameEntity, int message);

// die 
void Koopa_die(Koopa this);

// shake it
void Koopa_shake(Koopa this);

// be thrown over an axis and a direction
void Koopa_beThrown(Koopa this, int axis, s8 direction);

// tell whether it is ready to attack mario
int Koopa_canAttack(Koopa this);

#endif /*HALLOWKOOPALOGIC_H*/