/* VbJaEngine: bitmap graphics engine for the Nintendo Virtual Boy 
 * 
 * Copyright (C) 2007 Jorge Eremiev
 * jorgech3@gmail.com
 * 
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA
 */



/* ---------------------------------------------------------------------------------------------------------
 * ---------------------------------------------------------------------------------------------------------
 * ---------------------------------------------------------------------------------------------------------
 * 												INCLUDES
 * ---------------------------------------------------------------------------------------------------------
 * ---------------------------------------------------------------------------------------------------------
 * ---------------------------------------------------------------------------------------------------------
 */


#include "KoopaWalking.h"
#include "Koopa.h"


/* ---------------------------------------------------------------------------------------------------------
 * ---------------------------------------------------------------------------------------------------------
 * ---------------------------------------------------------------------------------------------------------
 * 												PROTOTYPES
 * ---------------------------------------------------------------------------------------------------------
 * ---------------------------------------------------------------------------------------------------------
 * ---------------------------------------------------------------------------------------------------------
 */


// class's constructor
static void KoopaWalking_constructor(KoopaWalking this);

// class's destructor
static void KoopaWalking_destructor(KoopaWalking this);

// state's enter
static void KoopaWalking_enter(KoopaWalking this, void* owner);

// state's execute
static void KoopaWalking_execute(KoopaWalking this, void* owner);

// state's enter
static void KoopaWalking_exit(KoopaWalking this, void* owner);

// state's pause
static void KoopaWalking_pause(KoopaWalking this, void* owner){}

// state's resume
static void KoopaWalking_resume(KoopaWalking this, void* owner);

// state's on message
static u16 KoopaWalking_handleMessage(KoopaWalking this, void* owner, Telegram telegram);


/* ---------------------------------------------------------------------------------------------------------
 * ---------------------------------------------------------------------------------------------------------
 * ---------------------------------------------------------------------------------------------------------
 * 											CLASS'S DEFINITION
 * ---------------------------------------------------------------------------------------------------------
 * ---------------------------------------------------------------------------------------------------------
 * ---------------------------------------------------------------------------------------------------------
 */

__CLASS_DEFINITION(KoopaWalking, State);

/* ---------------------------------------------------------------------------------------------------------
 * ---------------------------------------------------------------------------------------------------------
 * ---------------------------------------------------------------------------------------------------------
 * 												CLASS'S METHODS
 * ---------------------------------------------------------------------------------------------------------
 * ---------------------------------------------------------------------------------------------------------
 * ---------------------------------------------------------------------------------------------------------
 */
//////////////////////////////////////////////////////////////////////////////////////////////////////////////
__SINGLETON(KoopaWalking);

//////////////////////////////////////////////////////////////////////////////////////////////////////////////
// class's constructor
static void KoopaWalking_constructor(KoopaWalking this){

	__STATE_CONSTRUCT(KoopaWalking, State);
}

//////////////////////////////////////////////////////////////////////////////////////////////////////////////
// class's destructor
static void KoopaWalking_destructor(KoopaWalking this){
	
	
	// allow another creation of this state
	__SINGLETON_DESTRUCT;	
}

//////////////////////////////////////////////////////////////////////////////////////////////////////////////
// state's enter
static void KoopaWalking_enter(KoopaWalking this, void* owner){

	Actor_stopMovement((Actor)owner);
	
	InGameEntity_setGap((InGameEntity)owner);
	
	// play animation
	Actor_playAnimation((Actor)owner, "Walk");

	
	Actor_startXMovement((Actor)owner, __UNIFORMMOVE, KOOPA_VELOCITY_X, KOOPA_ACCELERATION_X);
}


//////////////////////////////////////////////////////////////////////////////////////////////////////////////
// state's execute
static void KoopaWalking_execute(KoopaWalking this, void* owner){
	
	// update movement
	Koopa_walk((Koopa) owner);
}


//////////////////////////////////////////////////////////////////////////////////////////////////////////////
// state's exit 
static void KoopaWalking_exit(KoopaWalking this, void* owner){
	
}

//////////////////////////////////////////////////////////////////////////////////////////////////////////////
// state's resume
static void KoopaWalking_resume(KoopaWalking this, void* owner){

	Actor_stopMovement((Actor)owner);
	
	InGameEntity_setGap((InGameEntity)owner);
	
	// play animation
	Actor_playAnimation((Actor)owner, "Walk");

	
	Actor_startXMovement((Actor)owner, __UNIFORMMOVE, KOOPA_VELOCITY_X, KOOPA_ACCELERATION_X);

	
}

//////////////////////////////////////////////////////////////////////////////////////////////////////////////
// state's on message
static u16 KoopaWalking_handleMessage(KoopaWalking this, void* owner, Telegram telegram){


	
	switch(Telegram_getMessage(telegram)){
	
		case kNoObjectBelow:

			Actor_fall((Actor)owner);			
			return true;
			break;
	

		case kCollisionXY:
		case kCollisionX:
		case kCollisionY:
		case kCollisionZ:

			// process the collision
			return Koopa_processCollision((Koopa)owner, telegram);				
			break;
	}
	return false;
}

