/* VbJaEngine: bitmap graphics engine for the Nintendo Virtual Boy 
 * 
 * Copyright (C) 2007 Jorge Eremiev
 * jorgech3@gmail.com
 * 
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA
 */



/* ---------------------------------------------------------------------------------------------------------
 * ---------------------------------------------------------------------------------------------------------
 * ---------------------------------------------------------------------------------------------------------
 * 												INCLUDES
 * ---------------------------------------------------------------------------------------------------------
 * ---------------------------------------------------------------------------------------------------------
 * ---------------------------------------------------------------------------------------------------------
 */

#include "MarioDead.h"
#include "MarioIdle.h"
#include "Mario.h"
#include "../../worlds/GameWorld.h"

/* ---------------------------------------------------------------------------------------------------------
 * ---------------------------------------------------------------------------------------------------------
 * ---------------------------------------------------------------------------------------------------------
 * 												PROTOTYPES
 * ---------------------------------------------------------------------------------------------------------
 * ---------------------------------------------------------------------------------------------------------
 * ---------------------------------------------------------------------------------------------------------
 */


// class's constructor
static void MarioDead_constructor(MarioDead this);

// class's destructor
static void MarioDead_destructor(MarioDead this);

// state's enter
static void MarioDead_enter(MarioDead this, void* owner);

// state's execute
static void MarioDead_execute(MarioDead this, void* owner);

// state's enter
static void MarioDead_exit(MarioDead this, void* owner);

// state's pause
static void MarioDead_pause(MarioDead this, void* owner){}

// state's resume
static void MarioDead_resume(MarioDead this, void* owner){}

// state's on message
static u16 MarioDead_handleMessage(MarioDead this, void* owner, Telegram telegram);


/* ---------------------------------------------------------------------------------------------------------
 * ---------------------------------------------------------------------------------------------------------
 * ---------------------------------------------------------------------------------------------------------
 * 											CLASS'S DEFINITION
 * ---------------------------------------------------------------------------------------------------------
 * ---------------------------------------------------------------------------------------------------------
 * ---------------------------------------------------------------------------------------------------------
 */

__CLASS_DEFINITION(MarioDead, State);

/* ---------------------------------------------------------------------------------------------------------
 * ---------------------------------------------------------------------------------------------------------
 * ---------------------------------------------------------------------------------------------------------
 * 												CLASS'S METHODS
 * ---------------------------------------------------------------------------------------------------------
 * ---------------------------------------------------------------------------------------------------------
 * ---------------------------------------------------------------------------------------------------------
 */
//////////////////////////////////////////////////////////////////////////////////////////////////////////////
__SINGLETON(MarioDead);

//////////////////////////////////////////////////////////////////////////////////////////////////////////////
// class's constructor
static void MarioDead_constructor(MarioDead this){

	__STATE_CONSTRUCT(MarioDead, State);
}

//////////////////////////////////////////////////////////////////////////////////////////////////////////////
// class's destructor
static void MarioDead_destructor(MarioDead this){
	
	
	// allow another creation of this state
	__SINGLETON_DESTRUCT;	
}

//////////////////////////////////////////////////////////////////////////////////////////////////////////////
// state's enter
static void MarioDead_enter(MarioDead this, void* owner){

	Direction direction = InGameEntity_getDirection((InGameEntity)owner);
	direction.z = __NEAR;
	InGameEntity_setDirection((InGameEntity)owner, direction);
	
	// stop movement
	Actor_stopMovement((Actor)owner);

}


//////////////////////////////////////////////////////////////////////////////////////////////////////////////
// state's execute
static void MarioDead_execute(MarioDead this, void* owner){

	// if die time elapsed
	if(MARIO_TIME_TO_DIE < Clock_getTime(_inGameClock) - Mario_getActionTime((Mario)owner)){
		
		if(!(__YAXIS & Actor_isMoving((Actor)owner))){
			
			// play animation
			Actor_playAnimation((Actor)owner, "HitFront");
		
			// and jump
			Actor_jump((Actor)owner, MARIO_VELOCITY_Y, MARIO_ACCELERATION_Y);
			
			// move over z axis
			Actor_startZMovement((Actor)owner, ~(__ACCELMOVEX | __RETARMOVEX),
					DEAD_VELOCITY_Z, DEAD_ACCELERATION_Z);
			
		}
		
	}
	

	// update movement 
	Actor_move((Actor)owner);
	
}


//////////////////////////////////////////////////////////////////////////////////////////////////////////////
// state's exit 
static void MarioDead_exit(MarioDead this, void* owner){
	
}

//////////////////////////////////////////////////////////////////////////////////////////////////////////////
// state's on message
static u16 MarioDead_handleMessage(MarioDead this, void* owner, Telegram telegram){

	// if the botton of the screen has been reached
	if(kFloorReached == Telegram_getMessage(telegram)){
	
		// inform the game that I'm dead
		GameWorld_marioIsDead(GameWorld_getInstance());

	}

	return false;
}

