/* VbJaEngine: bitmap graphics engine for the Nintendo Virtual Boy 
 * 
 * Copyright (C) 2007 Jorge Eremiev
 * jorgech3@gmail.com
 * 
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA
 */



/* ---------------------------------------------------------------------------------------------------------
 * ---------------------------------------------------------------------------------------------------------
 * ---------------------------------------------------------------------------------------------------------
 * 												INCLUDES
 * ---------------------------------------------------------------------------------------------------------
 * ---------------------------------------------------------------------------------------------------------
 * ---------------------------------------------------------------------------------------------------------
 */

#include "MarioWin.h"
#include "MarioIdle.h"
#include "MarioDead.h"
#include "Mario.h"


/* ---------------------------------------------------------------------------------------------------------
 * ---------------------------------------------------------------------------------------------------------
 * ---------------------------------------------------------------------------------------------------------
 * 												PROTOTYPES
 * ---------------------------------------------------------------------------------------------------------
 * ---------------------------------------------------------------------------------------------------------
 * ---------------------------------------------------------------------------------------------------------
 */


// class's constructor
static void MarioWin_constructor(MarioWin this);

// class's destructor
static void MarioWin_destructor(MarioWin this);

// state's enter
static void MarioWin_enter(MarioWin this, void* owner);

// state's execute
static void MarioWin_execute(MarioWin this, void* owner);

// state's exit
static void MarioWin_exit(MarioWin this, void* owner);

// state's pause
static void MarioWin_pause(MarioWin this, void* owner){}

// state's resume
static void MarioWin_resume(MarioWin this, void* owner){}

// state's on message
static u16 MarioWin_handleMessage(MarioWin this, void* owner, Telegram telegram);


/* ---------------------------------------------------------------------------------------------------------
 * ---------------------------------------------------------------------------------------------------------
 * ---------------------------------------------------------------------------------------------------------
 * 											CLASS'S DEFINITION
 * ---------------------------------------------------------------------------------------------------------
 * ---------------------------------------------------------------------------------------------------------
 * ---------------------------------------------------------------------------------------------------------
 */

__CLASS_DEFINITION(MarioWin, State);

/* ---------------------------------------------------------------------------------------------------------
 * ---------------------------------------------------------------------------------------------------------
 * ---------------------------------------------------------------------------------------------------------
 * 												CLASS'S METHODS
 * ---------------------------------------------------------------------------------------------------------
 * ---------------------------------------------------------------------------------------------------------
 * ---------------------------------------------------------------------------------------------------------
 */
//////////////////////////////////////////////////////////////////////////////////////////////////////////////
__SINGLETON(MarioWin);

//////////////////////////////////////////////////////////////////////////////////////////////////////////////
// class's constructor
static void MarioWin_constructor(MarioWin this){

	__STATE_CONSTRUCT(MarioWin, State);
}

//////////////////////////////////////////////////////////////////////////////////////////////////////////////
// class's destructor
static void MarioWin_destructor(MarioWin this){
	
	
	// allow another creation of this state
	__SINGLETON_DESTRUCT;	
}

//////////////////////////////////////////////////////////////////////////////////////////////////////////////
// state's enter
static void MarioWin_enter(MarioWin this, void* owner){
	
	Actor_stopMovement((Actor)owner);
	
	Actor_startXMovement((Actor)owner, __UNIFORMMOVE, MARIO_VELOCITY_X, 0);
	
	Actor_playAnimation((Actor)owner, "Walk");
}


//////////////////////////////////////////////////////////////////////////////////////////////////////////////
// state's execute
static void MarioWin_execute(MarioWin this, void* owner){
	
	// update movement over the bridge
	Mario_moveOnWin((Mario) owner);

}


//////////////////////////////////////////////////////////////////////////////////////////////////////////////
// state's exit 
static void MarioWin_exit(MarioWin this, void* owner){
	
}

//////////////////////////////////////////////////////////////////////////////////////////////////////////////
// state's on message
static u16 MarioWin_handleMessage(MarioWin this, void* owner, Telegram telegram){


	switch(Telegram_getMessage(telegram)){

		case kCollisionXY:
		case kCollisionX:
		case kCollisionY:
		case kCollisionZ:

			
			return Mario_processCollision((Mario)owner, telegram);				
			break;
	}
	
	return false;
}

