/* VbJaEngine: bitmap graphics engine for the Nintendo Virtual Boy 
 * 
 * Copyright (C) 2007 Jorge Eremiev
 * jorgech3@gmail.com
 * 
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA
 */

/* ---------------------------------------------------------------------------------------------------------
 * ---------------------------------------------------------------------------------------------------------
 * ---------------------------------------------------------------------------------------------------------
 * 												INCLUDES
 * ---------------------------------------------------------------------------------------------------------
 * ---------------------------------------------------------------------------------------------------------
 * ---------------------------------------------------------------------------------------------------------
 */

#include "VbJaeScreen.h"
#include "TitleScreen.h"
#include "../objects.h"
#include "../../macros.h"

/* ---------------------------------------------------------------------------------------------------------
 * ---------------------------------------------------------------------------------------------------------
 * ---------------------------------------------------------------------------------------------------------
 * 												PROTOTYPES
 * ---------------------------------------------------------------------------------------------------------
 * ---------------------------------------------------------------------------------------------------------
 * ---------------------------------------------------------------------------------------------------------
 */

static void VbJaeScreen_destructor(VbJaeScreen this);

// class's constructor
static void VbJaeScreen_constructor(VbJaeScreen this);


// state's enter
static void VbJaeScreen_enter(VbJaeScreen this, void* owner);

// state's execute
static void VbJaeScreen_execute(VbJaeScreen this, void* owner);

// state's exit
static void VbJaeScreen_exit(VbJaeScreen this, void* owner);

// state's pause
static void VbJaeScreen_pause(VbJaeScreen this, void* owner){}

// state's resume
static void VbJaeScreen_resume(VbJaeScreen this, void* owner){}


// state's on message
static u16 VbJaeScreen_handleMessage(VbJaeScreen this, void* owner, Telegram telegram);



/* ---------------------------------------------------------------------------------------------------------
 * ---------------------------------------------------------------------------------------------------------
 * ---------------------------------------------------------------------------------------------------------
 * 											CLASS'S DEFINITION
 * ---------------------------------------------------------------------------------------------------------
 * ---------------------------------------------------------------------------------------------------------
 * ---------------------------------------------------------------------------------------------------------
 */


	
__CLASS_DEFINITION(VbJaeScreen, State);

/* ---------------------------------------------------------------------------------------------------------
 * ---------------------------------------------------------------------------------------------------------
 * ---------------------------------------------------------------------------------------------------------
 * 												CLASS'S METHODS
 * ---------------------------------------------------------------------------------------------------------
 * ---------------------------------------------------------------------------------------------------------
 * ---------------------------------------------------------------------------------------------------------
 */

/* ---------------------------------------------------------------------------------------------------------*/
// it's a singleton
__SINGLETON_DYNAMIC(VbJaeScreen);


/* ---------------------------------------------------------------------------------------------------------*/
// class's constructor
static void VbJaeScreen_constructor(VbJaeScreen this){
		
	if(this){		

		__STATE_CONSTRUCT(VbJaeScreen, State);
	}	
}

/* ---------------------------------------------------------------------------------------------------------*/
// class's destructor
static void VbJaeScreen_destructor(VbJaeScreen this){
	
	// allow another creation of this state
	__SINGLETON_DESTRUCT;	
	
	// destroy the super object
	__DEALLOCATE(State);	
}

/* ---------------------------------------------------------------------------------------------------------*/
// state's enter
static void VbJaeScreen_enter(VbJaeScreen this, void* owner){
	
	// make a fade out
	//Game_FXFadeOut(Game_getInstance(),FADEDELAY);
	Game_FXFadeOut(Game_getInstance(),10);
	
	//clear char and bgmap memory
	vbClearScreen();
	
	// reset the engine state
	Game_reset(Game_getInstance(), true);
	
	
	//load world entities	
	Stage_load(Game_getStage(Game_getInstance()), &VBJAE_WR);
	
	
	//call the render to setup world entries
	Game_render(Game_getInstance());
	 
	// reset the global clock
	//Clock_reset(Game_getClock(Game_getInstance()));
	
	// turn back the background
	VIP_REGS[BKCOL] = 0x00;
	
	// make a fade in
	Game_FXFadeIn(Game_getInstance(), FADEDELAY);	
}


/* ---------------------------------------------------------------------------------------------------------*/
// state's execute
static void VbJaeScreen_execute(VbJaeScreen this, void* owner){

	//update game world objects (characters, textbox, scroll brackgrounds)
	Stage_update(Game_getStage(Game_getInstance()));
}

/* ---------------------------------------------------------------------------------------------------------*/
// state's exit 
static void VbJaeScreen_exit(VbJaeScreen this, void* owner){
	
	// destroy the state
	__DELETE(this);
}

/* ---------------------------------------------------------------------------------------------------------*/
// state's on message
static u16 VbJaeScreen_handleMessage(VbJaeScreen this, void* owner, Telegram telegram){

	
	// process message
	switch(Telegram_getMessage(telegram)){
	
		case kKeyPressed:	
			
			//if start button pressed
			if(vbNewKeyPressed(K_STA)){
				
				//if(Clock_getTime(Game_getClock(Game_getInstance())) > SHOWDELAY){
				if(Clock_getTime(Game_getClock(Game_getInstance())) > 0){
					
					//setup title _SC	
					Game_setState(Game_getInstance(), (State)TitleScreen_getInstance());
					//Game_setState(Game_getInstance(), (State)PVBCCScreen_getInstance());
				}
				
				return true;
			}
	
			
			break;
	}

	return false;
}
