/* VbJaEngine: bitmap graphics engine for the Nintendo Virtual Boy 
 * 
 * Copyright (C) 2007 Jorge Eremiev
 * jorgech3@gmail.com
 * 
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA
 */



/* ---------------------------------------------------------------------------------------------------------
 * ---------------------------------------------------------------------------------------------------------
 * ---------------------------------------------------------------------------------------------------------
 * 												INCLUDES
 * ---------------------------------------------------------------------------------------------------------
 * ---------------------------------------------------------------------------------------------------------
 * ---------------------------------------------------------------------------------------------------------
 */

#include "KoopaBouncing.h"
#include "KoopaStunned.h"
#include "Koopa.h"


/* ---------------------------------------------------------------------------------------------------------
 * ---------------------------------------------------------------------------------------------------------
 * ---------------------------------------------------------------------------------------------------------
 * 												PROTOTYPES
 * ---------------------------------------------------------------------------------------------------------
 * ---------------------------------------------------------------------------------------------------------
 * ---------------------------------------------------------------------------------------------------------
 */


// class's constructor
static void KoopaBouncing_constructor(KoopaBouncing this);

// class's destructor
static void KoopaBouncing_destructor(KoopaBouncing this);

// state's enter
static void KoopaBouncing_enter(KoopaBouncing this, void* owner);

// state's execute
static void KoopaBouncing_execute(KoopaBouncing this, void* owner);

// state's enter
static void KoopaBouncing_exit(KoopaBouncing this, void* owner);

// state's pause
static void KoopaBouncing_pause(KoopaBouncing this, void* owner){}

// state's resume
static void KoopaBouncing_resume(KoopaBouncing this, void* owner);

// state's on message
static u16 KoopaBouncing_handleMessage(KoopaBouncing this, void* owner, Telegram telegram);

/* ---------------------------------------------------------------------------------------------------------
 * ---------------------------------------------------------------------------------------------------------
 * ---------------------------------------------------------------------------------------------------------
 * 											CLASS'S DEFINITION
 * ---------------------------------------------------------------------------------------------------------
 * ---------------------------------------------------------------------------------------------------------
 * ---------------------------------------------------------------------------------------------------------
 */

__CLASS_DEFINITION(KoopaBouncing, State);

/* ---------------------------------------------------------------------------------------------------------
 * ---------------------------------------------------------------------------------------------------------
 * ---------------------------------------------------------------------------------------------------------
 * 												CLASS'S METHODS
 * ---------------------------------------------------------------------------------------------------------
 * ---------------------------------------------------------------------------------------------------------
 * ---------------------------------------------------------------------------------------------------------
 */
//////////////////////////////////////////////////////////////////////////////////////////////////////////////
__SINGLETON(KoopaBouncing);

//////////////////////////////////////////////////////////////////////////////////////////////////////////////
// class's constructor
static void KoopaBouncing_constructor(KoopaBouncing this){

	__STATE_CONSTRUCT(KoopaBouncing, State);
}

//////////////////////////////////////////////////////////////////////////////////////////////////////////////
// class's destructor
static void KoopaBouncing_destructor(KoopaBouncing this){
	
	
	// allow another creation of this state
	__SINGLETON_DESTRUCT;	
}

//////////////////////////////////////////////////////////////////////////////////////////////////////////////
// state's enter
static void KoopaBouncing_enter(KoopaBouncing this, void* owner){

	// must update koopa's gap
	Koopa_setGap((Koopa)owner);
}


//////////////////////////////////////////////////////////////////////////////////////////////////////////////
// state's execute
static void KoopaBouncing_execute(KoopaBouncing this, void* owner){
	
	// update movement
	Koopa_slide((Koopa) owner);
}


//////////////////////////////////////////////////////////////////////////////////////////////////////////////
// state's exit 
static void KoopaBouncing_exit(KoopaBouncing this, void* owner){
	
}

//////////////////////////////////////////////////////////////////////////////////////////////////////////////
// state's resume
static void KoopaBouncing_resume(KoopaBouncing this, void* owner){

	//Actor_stopMovement((Actor)owner);
	//Actor_startXMovement((Actor)owner, __UNIFORMMOVE, KOOPA_BOUNCE_VELOCITY_X, KOOPA_ACCELERATION_X);
	Actor_resetTimers((Actor)owner);
}

//////////////////////////////////////////////////////////////////////////////////////////////////////////////
// state's on message
static u16 KoopaBouncing_handleMessage(KoopaBouncing this, void* owner, Telegram telegram){


	
	switch(Telegram_getMessage(telegram)){
	
		case kNoObjectBelow:		
	
			if(!(__ZAXIS & Actor_isMoving((Actor)owner))){
				// must fall
				Actor_fall((Actor)owner);
			}
			return true;
			break;
	

		case kCollisionXY:
		case kCollisionX:
		case kCollisionY:
		case kCollisionZ:

			// process the collision
			return Koopa_processCollision((Koopa)owner, telegram);				
			break;
	}
	return false;
}

