/* VbJaEngine: bitmap graphics engine for the Nintendo Virtual Boy 
 * 
 * Copyright (C) 2007 Jorge Eremiev
 * jorgech3@gmail.com
 * 
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA
 */



/* ---------------------------------------------------------------------------------------------------------
 * ---------------------------------------------------------------------------------------------------------
 * ---------------------------------------------------------------------------------------------------------
 * 												INCLUDES
 * ---------------------------------------------------------------------------------------------------------
 * ---------------------------------------------------------------------------------------------------------
 * ---------------------------------------------------------------------------------------------------------
 */


#include "KoopaStunned.h"
#include "KoopaWalking.h"
#include "Koopa.h"


/* ---------------------------------------------------------------------------------------------------------
 * ---------------------------------------------------------------------------------------------------------
 * ---------------------------------------------------------------------------------------------------------
 * 												PROTOTYPES
 * ---------------------------------------------------------------------------------------------------------
 * ---------------------------------------------------------------------------------------------------------
 * ---------------------------------------------------------------------------------------------------------
 */


// class's constructor
static void KoopaStunned_constructor(KoopaStunned this);

// class's destructor
static void KoopaStunned_destructor(KoopaStunned this);

// state's enter
static void KoopaStunned_enter(KoopaStunned this, void* owner);

// state's execute
static void KoopaStunned_execute(KoopaStunned this, void* owner);

// state's enter
static void KoopaStunned_exit(KoopaStunned this, void* owner);

// state's pause
static void KoopaStunned_pause(KoopaStunned this, void* owner){}

// state's resume
static void KoopaStunned_resume(KoopaStunned this, void* owner){}



// state's on message
static u16 KoopaStunned_handleMessage(KoopaStunned this, void* owner, Telegram telegram);


/* ---------------------------------------------------------------------------------------------------------
 * ---------------------------------------------------------------------------------------------------------
 * ---------------------------------------------------------------------------------------------------------
 * 											CLASS'S DEFINITION
 * ---------------------------------------------------------------------------------------------------------
 * ---------------------------------------------------------------------------------------------------------
 * ---------------------------------------------------------------------------------------------------------
 */

__CLASS_DEFINITION(KoopaStunned, State);

/* ---------------------------------------------------------------------------------------------------------
 * ---------------------------------------------------------------------------------------------------------
 * ---------------------------------------------------------------------------------------------------------
 * 												CLASS'S METHODS
 * ---------------------------------------------------------------------------------------------------------
 * ---------------------------------------------------------------------------------------------------------
 * ---------------------------------------------------------------------------------------------------------
 */
//////////////////////////////////////////////////////////////////////////////////////////////////////////////
__SINGLETON(KoopaStunned);

//////////////////////////////////////////////////////////////////////////////////////////////////////////////
// class's constructor
static void KoopaStunned_constructor(KoopaStunned this){

	__STATE_CONSTRUCT(KoopaStunned, State);
}

//////////////////////////////////////////////////////////////////////////////////////////////////////////////
// class's destructor
static void KoopaStunned_destructor(KoopaStunned this){
	
	
	// allow another creation of this state
	__SINGLETON_DESTRUCT;	
}

//////////////////////////////////////////////////////////////////////////////////////////////////////////////
// state's enter
static void KoopaStunned_enter(KoopaStunned this, void* owner){
	
	VBVec3D position = Entity_getPosition((Entity)owner);
	//position.y -= 1;
	Entity_setPosition((Entity)owner, position);
	
	Koopa_setGap((Koopa)owner);
	
	Actor_stopMovement((Actor)owner);
}


//////////////////////////////////////////////////////////////////////////////////////////////////////////////
// state's execute
static void KoopaStunned_execute(KoopaStunned this, void* owner){
	
	u32 currentTime = Clock_getTime(_inGameClock);
	u32 koopaActionTime = Enemy_getActionTime((Enemy)owner);
	
	
	if(KOOPA_RECOVER_TIME < currentTime - koopaActionTime){
		
		if((KOOPA_RECOVER_TIME * 1.5f) > currentTime - koopaActionTime){
			
			// make koopa shake
			Koopa_shake((Koopa)owner);
			return;
		}
		
		
		if(!AnimatedSprite_isPlaying((AnimatedSprite)Entity_getSprite((Entity)owner))){
			
			// walk again
			StateMachine_swapState(Actor_getStateMachine((Actor)owner), (State)KoopaWalking_getInstance());

			// play animation
			Actor_playAnimation((Actor)owner, "Recover");
			
			// play animation
			Actor_playAnimation((Actor)owner, "Recover");
		}
	}
	
	// move the character
	if(__YAXIS & Actor_isMoving((Actor)owner)){
		
		Actor_keepYMovement((Actor)owner);

	}
	
}


//////////////////////////////////////////////////////////////////////////////////////////////////////////////
// state's exit 
static void KoopaStunned_exit(KoopaStunned this, void* owner){
	
	// if my shape was removed by mario
	Koopa_registerShape((Koopa)owner);
}

//////////////////////////////////////////////////////////////////////////////////////////////////////////////
// state's on message
static u16 KoopaStunned_handleMessage(KoopaStunned this, void* owner, Telegram telegram){

	switch(Telegram_getMessage(telegram)){
	
		case kNoObjectBelow:		
	
			// must fall
			Actor_fall((Actor)owner);
			
			return true;
			break;
	

		case kCollisionY:
			

			// if was falling
			if(__YAXIS & Actor_isMoving((Actor)owner)){
			
				InGameEntity inGameEntity = (InGameEntity)Telegram_getExtraInfo(telegram);
				
				// aling to below object
				Actor_alignY((Actor)owner, (InGameEntity)inGameEntity, 1);
				
				// finally stop the movement
				Actor_stopYMovement((Actor)owner);

				// save object below me
				Actor_setObjectBelow((Actor)owner, (InGameEntity)inGameEntity);
				
				return true;
			}
			//return Koopa_checkIfTurn((Koopa)owner, (InGameEntity)  Telegram_getExtraInfo(telegram), Telegram_getMessage(telegram));
			// process the collision
			//return Koopa_processCollision((Koopa)owner, telegram);				
			break;
	}

	return false;
}

