/* VbJaEngine: bitmap graphics engine for the Nintendo Virtual Boy 
 * 
 * Copyright (C) 2007 Jorge Eremiev
 * jorgech3@gmail.com
 * 
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA
 */

/*---------------------------------INCLUDES--------------------------------*/
#include <libgccvb.h>
#include "Koopa.h"



extern BYTE KOOPA0_CH[];
extern BYTE KOOPA0_MP[];


// a function which defines the frames to play
AnimationFunction KOOPA_WALKING_ANIM = {
	
	// function's name
	"Walk",

	// number of frames of this animation function
	4,
	
	
	// frames to play in animation
	{1, 0, 2, 0},
	
	
	// number of cicles a frame of animation is displayed
	10 * __FPS_ANIM_FACTOR,
	
	// whether to play it in loop or not
	true,
	
	// method to call function completion
	NULL,
	
};

AnimationFunction KOOPA_STUNNED_ANIM = {
	
	// function's name
	"Stunned",

	// number of frames of this animation function
	4,
	
	// frames to play in animation
	{5, 6, 7, 8},
	
	// number of cicles a frame of animation is displayed
	5 * __FPS_ANIM_FACTOR,
	
	// whether to play it in loop or not
	false,
	
	// method to call function completion
	NULL,
	
};

AnimationFunction KOOPA_RECOVER_ANIM = {
	
	// function's name
	"Recover",

	// number of frames of this animation function
	3,
	
	// frames to play in animation
	{7, 6, 5,},
	
	
	// number of cicles a frame of animation is displayed
	8 * __FPS_ANIM_FACTOR,
	
	// whether to play it in loop or not
	false,
	
	// method to call on function completion
	Koopa_recover,
	
};

AnimationFunction KOOPA_TURN_ANIM = {
	
	// function's name
	"Turn",

	// number of frames of this animation function
	2,
	
	// frames to play in animation
	{4, 3,},
	
	
	// number of cicles a frame of animation is displayed
	5 * __FPS_ANIM_FACTOR,
	
	// whether to play it in loop or not
	false,
	
	// method to call function completion
	Koopa_turnComplete,
	
};


// a function which defines the frames to play
AnimationFunction KOOPA_ATTACK_ANIM = {
	
	// function's name
	"Attack",

	// number of frames of this animation function
	1,
	
	// frames to play in animation
	{5},
	
	// number of cicles a frame of animation is displayed
	0,
	
	// whether to play it in loop or not
	false,
	
	// method to call function completion
	NULL,
	
};

// an animation definition
AnimationDescription KOOPA_ANIM = {
	
	// number of animation frames
	9,
	
	// animation functions
	{
		&KOOPA_WALKING_ANIM,
		&KOOPA_STUNNED_ANIM,
		&KOOPA_RECOVER_ANIM,
		&KOOPA_TURN_ANIM,
		&KOOPA_ATTACK_ANIM,
		NULL,
	}
	
};


//////////////////////////////////////////////////////////////////////////////////////////////////////////////
TextureDefinition KOOPA0_TX = {
		
		// Chargroup
		{
				// chDefinition,				 
				KOOPA0_CH,	
				
				// numChars,
				9,
				
				// allocation type
				__ANIMATED_SHARED
		},
		
		// bgmap definition
		KOOPA0_MP,
		
		// cols (max 48)
		3,
		
		// rows (max 28)
		3,
		
		//pallet number,
		1
};

ActorDefinition KOOPA0_MC = {
		{
			{
				// object's class			   
				__TYPE(Koopa),  
		
				// Sprite
				{
					// the texture
					&KOOPA0_TX,
					
					// bgmap mode ( BGMAP, AFFINE, H-BIAS)
					WRLD_AFFINE,
					
					// display mode
					WRLD_ON,
				},
			},
			
			// deep
			5,
			
			// friction factor
			5.5,
			
			//collision detection gap			
			//up,	down,	left,	right,
			{2,		1,		0,		0,},		
			
			// in game type
			kKoopa,
			
		},
		
		// pointer to the animation definition for the character
		&KOOPA_ANIM,
};



