/* VbJaEngine: bitmap graphics engine for the Nintendo Virtual Boy 
 * 
 * Copyright (C) 2007 Jorge Eremiev
 * jorgech3@gmail.com
 * 
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA
 */



/* ---------------------------------------------------------------------------------------------------------
 * ---------------------------------------------------------------------------------------------------------
 * ---------------------------------------------------------------------------------------------------------
 * 												INCLUDES
 * ---------------------------------------------------------------------------------------------------------
 * ---------------------------------------------------------------------------------------------------------
 * ---------------------------------------------------------------------------------------------------------
 */

#include "MarioMoving.h"
#include "MarioIdle.h"
#include "MarioDead.h"
#include "Mario.h"


/* ---------------------------------------------------------------------------------------------------------
 * ---------------------------------------------------------------------------------------------------------
 * ---------------------------------------------------------------------------------------------------------
 * 												PROTOTYPES
 * ---------------------------------------------------------------------------------------------------------
 * ---------------------------------------------------------------------------------------------------------
 * ---------------------------------------------------------------------------------------------------------
 */


// class's constructor
static void MarioMoving_constructor(MarioMoving this);

// class's destructor
static void MarioMoving_destructor(MarioMoving this);

// state's enter
static void MarioMoving_enter(MarioMoving this, void* owner);

// state's execute
static void MarioMoving_execute(MarioMoving this, void* owner);

// state's exit
static void MarioMoving_exit(MarioMoving this, void* owner);

// state's pause
static void MarioMoving_pause(MarioMoving this, void* owner){}

// state's resume
static void MarioMoving_resume(MarioMoving this, void* owner){}

// state's on message
static u16 MarioMoving_handleMessage(MarioMoving this, void* owner, Telegram telegram);


/* ---------------------------------------------------------------------------------------------------------
 * ---------------------------------------------------------------------------------------------------------
 * ---------------------------------------------------------------------------------------------------------
 * 											CLASS'S DEFINITION
 * ---------------------------------------------------------------------------------------------------------
 * ---------------------------------------------------------------------------------------------------------
 * ---------------------------------------------------------------------------------------------------------
 */

__CLASS_DEFINITION(MarioMoving, State);

/* ---------------------------------------------------------------------------------------------------------
 * ---------------------------------------------------------------------------------------------------------
 * ---------------------------------------------------------------------------------------------------------
 * 												CLASS'S METHODS
 * ---------------------------------------------------------------------------------------------------------
 * ---------------------------------------------------------------------------------------------------------
 * ---------------------------------------------------------------------------------------------------------
 */
//////////////////////////////////////////////////////////////////////////////////////////////////////////////
__SINGLETON(MarioMoving);

//////////////////////////////////////////////////////////////////////////////////////////////////////////////
// class's constructor
static void MarioMoving_constructor(MarioMoving this){

	__STATE_CONSTRUCT(MarioMoving, State);
}

//////////////////////////////////////////////////////////////////////////////////////////////////////////////
// class's destructor
static void MarioMoving_destructor(MarioMoving this){
	
	
	// allow another creation of this state
	__SINGLETON_DESTRUCT;	
}

//////////////////////////////////////////////////////////////////////////////////////////////////////////////
// state's enter
static void MarioMoving_enter(MarioMoving this, void* owner){
	
	Mario_setBridge((Mario)owner, NULL);
	
	Mario_startMoving((Mario)owner);
	
	Mario_setGap((Mario)owner);
}


//////////////////////////////////////////////////////////////////////////////////////////////////////////////
// state's execute
static void MarioMoving_execute(MarioMoving this, void* owner){
	
	// update movement
	Mario_move((Mario) owner);
		
}


//////////////////////////////////////////////////////////////////////////////////////////////////////////////
// state's exit 
static void MarioMoving_exit(MarioMoving this, void* owner){
	
}

//////////////////////////////////////////////////////////////////////////////////////////////////////////////
// state's on message
static u16 MarioMoving_handleMessage(MarioMoving this, void* owner, Telegram telegram){

	

	switch(Telegram_getMessage(telegram)){

		// if the botton of the screen has been reached
		case kFloorReached:
	
			Mario_fallDead((Mario)owner);

			Actor_stopYMovement(owner);
			break;
			
		case kNoObjectBelow:

			Actor_fall((Actor)owner);			
			return true;
			break;
	

		case kCollisionXY:
		case kCollisionX:
		case kCollisionY:
		case kCollisionZ:

			return Mario_processCollision((Mario)owner, telegram);				
			break;
			
		case kKeyPressed:
			{
				u16 pressedKey = vbKeyPressed();
				
				if((K_LL | K_LR ) & pressedKey){
					
					Mario_checkDirection((Mario)owner, pressedKey);
					
				}
				
				if(K_A & pressedKey){
					
					Mario_jump((Mario)owner);			
				}
				
			}			
			return true;
			break;
			
	}
	return false;
	
}

