/* VbJaEngine: bitmap graphics engine for the Nintendo Virtual Boy 
 * 
 * Copyright (C) 2007 Jorge Eremiev
 * jorgech3@gmail.com
 * 
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA
 */



/* ---------------------------------------------------------------------------------------------------------
 * ---------------------------------------------------------------------------------------------------------
 * ---------------------------------------------------------------------------------------------------------
 * 												INCLUDES
 * ---------------------------------------------------------------------------------------------------------
 * ---------------------------------------------------------------------------------------------------------
 * ---------------------------------------------------------------------------------------------------------
 */

#include "MarioOnBridge.h"
#include "MarioIdle.h"
#include "MarioDead.h"
#include "Mario.h"


/* ---------------------------------------------------------------------------------------------------------
 * ---------------------------------------------------------------------------------------------------------
 * ---------------------------------------------------------------------------------------------------------
 * 												PROTOTYPES
 * ---------------------------------------------------------------------------------------------------------
 * ---------------------------------------------------------------------------------------------------------
 * ---------------------------------------------------------------------------------------------------------
 */


// class's constructor
static void MarioOnBridge_constructor(MarioOnBridge this);

// class's destructor
static void MarioOnBridge_destructor(MarioOnBridge this);

// state's enter
static void MarioOnBridge_enter(MarioOnBridge this, void* owner);

// state's execute
static void MarioOnBridge_execute(MarioOnBridge this, void* owner);

// state's exit
static void MarioOnBridge_exit(MarioOnBridge this, void* owner);

// state's pause
static void MarioOnBridge_pause(MarioOnBridge this, void* owner){}

// state's resume
static void MarioOnBridge_resume(MarioOnBridge this, void* owner){}

// state's on message
static u16 MarioOnBridge_handleMessage(MarioOnBridge this, void* owner, Telegram telegram);


/* ---------------------------------------------------------------------------------------------------------
 * ---------------------------------------------------------------------------------------------------------
 * ---------------------------------------------------------------------------------------------------------
 * 											CLASS'S DEFINITION
 * ---------------------------------------------------------------------------------------------------------
 * ---------------------------------------------------------------------------------------------------------
 * ---------------------------------------------------------------------------------------------------------
 */

__CLASS_DEFINITION(MarioOnBridge, State);

/* ---------------------------------------------------------------------------------------------------------
 * ---------------------------------------------------------------------------------------------------------
 * ---------------------------------------------------------------------------------------------------------
 * 												CLASS'S METHODS
 * ---------------------------------------------------------------------------------------------------------
 * ---------------------------------------------------------------------------------------------------------
 * ---------------------------------------------------------------------------------------------------------
 */
//////////////////////////////////////////////////////////////////////////////////////////////////////////////
__SINGLETON(MarioOnBridge);

//////////////////////////////////////////////////////////////////////////////////////////////////////////////
// class's constructor
static void MarioOnBridge_constructor(MarioOnBridge this){

	__STATE_CONSTRUCT(MarioOnBridge, State);
}

//////////////////////////////////////////////////////////////////////////////////////////////////////////////
// class's destructor
static void MarioOnBridge_destructor(MarioOnBridge this){
	
	
	// allow another creation of this state
	__SINGLETON_DESTRUCT;	
}

//////////////////////////////////////////////////////////////////////////////////////////////////////////////
// state's enter
static void MarioOnBridge_enter(MarioOnBridge this, void* owner){
	
	Actor_stopMovement((Actor)owner);
}


//////////////////////////////////////////////////////////////////////////////////////////////////////////////
// state's execute
static void MarioOnBridge_execute(MarioOnBridge this, void* owner){
	
	// update movement over the bridge
	Mario_moveOnBridge((Mario) owner);
}


//////////////////////////////////////////////////////////////////////////////////////////////////////////////
// state's exit 
static void MarioOnBridge_exit(MarioOnBridge this, void* owner){
	
}

//////////////////////////////////////////////////////////////////////////////////////////////////////////////
// state's on message
static u16 MarioOnBridge_handleMessage(MarioOnBridge this, void* owner, Telegram telegram){


	switch(Telegram_getMessage(telegram)){

		case kCollisionXY:
		case kCollisionX:
		case kCollisionY:
		case kCollisionZ:

			return Mario_processCollision((Mario)owner, telegram);				
			break;
	}
	
	return false;
}

