/* VbJaEngine: bitmap graphics engine for the Nintendo Virtual Boy 
 * 
 * Copyright (C) 2007 Jorge Eremiev
 * jorgech3@gmail.com
 * 
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA
 */



/* ---------------------------------------------------------------------------------------------------------
 * ---------------------------------------------------------------------------------------------------------
 * ---------------------------------------------------------------------------------------------------------
 * 												INCLUDES
 * ---------------------------------------------------------------------------------------------------------
 * ---------------------------------------------------------------------------------------------------------
 * ---------------------------------------------------------------------------------------------------------
 */


#include "PiranhaPlantIdle.h"
#include "PiranhaPlantMoving.h"
#include "PiranhaPlant.h"
#include "../mario/Mario.h"


/* ---------------------------------------------------------------------------------------------------------
 * ---------------------------------------------------------------------------------------------------------
 * ---------------------------------------------------------------------------------------------------------
 * 												PROTOTYPES
 * ---------------------------------------------------------------------------------------------------------
 * ---------------------------------------------------------------------------------------------------------
 * ---------------------------------------------------------------------------------------------------------
 */


// class's constructor
static void PiranhaPlantIdle_constructor(PiranhaPlantIdle this);

// class's destructor
static void PiranhaPlantIdle_destructor(PiranhaPlantIdle this);

// state's enter
static void PiranhaPlantIdle_enter(PiranhaPlantIdle this, void* owner);

// state's execute
static void PiranhaPlantIdle_execute(PiranhaPlantIdle this, void* owner);

// state's enter
static void PiranhaPlantIdle_exit(PiranhaPlantIdle this, void* owner);

// state's pause
static void PiranhaPlantIdle_pause(PiranhaPlantIdle this, void* owner){}

// state's resume
static void PiranhaPlantIdle_resume(PiranhaPlantIdle this, void* owner){}


// state's on message
static u16 PiranhaPlantIdle_handleMessage(PiranhaPlantIdle this, void* owner, Telegram telegram);


/* ---------------------------------------------------------------------------------------------------------
 * ---------------------------------------------------------------------------------------------------------
 * ---------------------------------------------------------------------------------------------------------
 * 											CLASS'S DEFINITION
 * ---------------------------------------------------------------------------------------------------------
 * ---------------------------------------------------------------------------------------------------------
 * ---------------------------------------------------------------------------------------------------------
 */

__CLASS_DEFINITION(PiranhaPlantIdle, State);

/* ---------------------------------------------------------------------------------------------------------
 * ---------------------------------------------------------------------------------------------------------
 * ---------------------------------------------------------------------------------------------------------
 * 												CLASS'S METHODS
 * ---------------------------------------------------------------------------------------------------------
 * ---------------------------------------------------------------------------------------------------------
 * ---------------------------------------------------------------------------------------------------------
 */
//////////////////////////////////////////////////////////////////////////////////////////////////////////////
__SINGLETON(PiranhaPlantIdle);

//////////////////////////////////////////////////////////////////////////////////////////////////////////////
// class's constructor
static void PiranhaPlantIdle_constructor(PiranhaPlantIdle this){

	__STATE_CONSTRUCT(PiranhaPlantIdle, State);
}

//////////////////////////////////////////////////////////////////////////////////////////////////////////////
// class's destructor
static void PiranhaPlantIdle_destructor(PiranhaPlantIdle this){
	
	
	// allow another creation of this state
	__SINGLETON_DESTRUCT;	
}

//////////////////////////////////////////////////////////////////////////////////////////////////////////////
// state's enter
static void PiranhaPlantIdle_enter(PiranhaPlantIdle this, void* owner){
	
	// do not move
	Actor_stopMovement((Actor)owner);

	Actor_playAnimation((Actor)owner, "Idle");
}


//////////////////////////////////////////////////////////////////////////////////////////////////////////////
// state's execute
static void PiranhaPlantIdle_execute(PiranhaPlantIdle this, void* owner){
	
	// check if mario distance to the plant is within range
	if(PIRANHA_PLANT_ATTACK_DISTANCE > vbjLengthSquared3D(
			Entity_getPosition((Entity)owner), Entity_getPosition((Entity)Mario_getInstance()))
	){
		StateMachine_swapState(Actor_getStateMachine((Actor)owner), (State)PiranhaPlantMoving_getInstance());
	}

}


//////////////////////////////////////////////////////////////////////////////////////////////////////////////
// state's exit 
static void PiranhaPlantIdle_exit(PiranhaPlantIdle this, void* owner){
	
}

//////////////////////////////////////////////////////////////////////////////////////////////////////////////
// state's on message
static u16 PiranhaPlantIdle_handleMessage(PiranhaPlantIdle this, void* owner, Telegram telegram){

	return false;
}

